@@ -30,7 +30,7 @@ int autoqueue(pli_t queue) {
  struct rpld_playlist_entry   * plent;
  struct rpld_playlist_entry   * copy;
  struct rpld_playlist_search    pls;
- struct rpld_playlist_pointer * plp, * plp_copy;
+ struct rpld_playlist_pointer * plp = NULL, * plp_copy = NULL;
 
  ROAR_DBG("autoqueue(void) = ?");
 
@@ -92,15 +92,9 @@ int autoqueue(pli_t queue) {
 
     memset(&pls, 0, sizeof(pls));
 
-#ifdef HAVE_LIB_UUID
     pls.type = RPLD_PL_ST_UUID;
 
-    uuid_copy(pls.subject.uuid, plent->uuid);
-#else
-    pls.type = RPLD_PL_ST_TRACKNUM_LONG;
-
-    pls.subject.long_tracknum = plent->global_tracknum;
-#endif
+    memcpy(pls.subject.uuid, plent->uuid, sizeof(pls.subject.uuid));
 
     plp_copy = rpld_plp_init(NULL, &pls);
     rpld_pointer_set(POINTER_CURRENT, queue, -1, plp_copy);
