@@ -26,10 +26,18 @@
 #ifndef _RPLD_CLIENT_H_
 #define _RPLD_CLIENT_H_
 
-#define RPLD_PROTO_UNUSED  -1
-#define RPLD_PROTO_SIMPLE   1
-#define RPLD_PROTO_MPD      2
+// Avoid problems libroar 1.0beta4.
+#ifndef ROAR_PROTO_RPLD
+#define ROAR_PROTO_RPLD 25
+#endif
 
+#ifndef ROAR_PROTO_MPD
+#define ROAR_PROTO_MPD 26
+#endif
+
+#define RPLD_PROTO_UNUSED   ROAR_PROTO_NONE
+
+#define RPLD_PROTOS_MAX     8
 #define RPLD_CLIENTS_MAX    8
 
 // access level:
@@ -49,13 +57,12 @@ enum rpld_client_state {
 };
 
 struct rpld_proto {
- int proto;
- const char * description;
- int (*set_proto)(int client, struct roar_vio_calls * vio, struct roar_buffer ** obuffer, void ** userdata);
- int (*unset_proto)(int client, struct roar_vio_calls * vio, struct roar_buffer ** obuffer, void ** userdata);
- int (*handle)(int client, struct roar_vio_calls * vio, struct roar_buffer ** obuffer, void ** userdata);
- int (*flush)(int client, struct roar_vio_calls * vio, struct roar_buffer ** obuffer, void ** userdata);
- int (*flushed)(int client, struct roar_vio_calls * vio, struct roar_buffer ** obuffer, void ** userdata);
+ const struct roar_keyval * para;
+ ssize_t paralen;
+ struct roar_dl_librarypara * pluginpara;
+ struct roar_dl_lhandle * lhandle;
+ 
+ const struct roar_dl_proto * impl;
 };
 
 void client_init(void);
@@ -69,6 +76,9 @@ enum rpld_client_acclev client_get_acclev(int id);
 int  client_set_acclev(int id, enum rpld_client_acclev acclev, int setter);
 int  client_inc_acclev(int id, int inc, int setter);
 
+int  client_set_name(int id, const char * name);
+const char * client_get_name(int id);
+
 int  client_set_playlist(int id, pli_t pl);
 pli_t client_get_playlist(int id);
 int  client_set_queue(int id, pli_t pl);
@@ -76,9 +86,11 @@ pli_t client_get_queue(int id);
 int  client_set_pointer(int id, int pointer, struct rpld_playlist_pointer * plp);
 struct rpld_playlist_pointer * client_get_pointer(int id, int pointer);
 
-int  client_handle(int id);
+int  client_handle(int id, int eventsa);
+int  client_get_eventsq(int id);
 
 int  client_str2proto(const char * str);
+const char * client_proto2str(const int proto);
 int  client_str2acclev(const char * str);
 
 #endif
