@@ -39,7 +39,7 @@ int ff_m3u_pl_export(struct fformat_handle * handle, struct roar_vio_calls * vio
   if ( *artist == 0 )
    artist = cur->meta.performer;
 
-  roar_vio_printf(vio, "#EXTINF:%i,%s - %s\n", cur->length, artist, cur->meta.title);
+  roar_vio_printf(vio, "#EXTINF:%lu,%s - %s\n", (unsigned long)cur->length, artist, cur->meta.title);
 
   roar_vio_printf(vio, "%s\n", cur->io.filename);
 
