@@ -46,23 +46,15 @@ static char *_strtok(char *str, const char *delim) {
 
 int ff_rpld_pl_export(struct fformat_handle * handle, struct roar_vio_calls * vio, struct rpld_playlist  * pl) {
  struct rpld_playlist_entry * plent = rpld_pl_get_first(pl);
-#ifdef HAVE_LIB_UUID
  char uuid[37];
-#endif
 
  (void)handle;
 
-#ifdef HAVE_LIB_UUID
- uuid_unparse(plent->uuid, uuid);
-#endif
+ roar_uuid2str(uuid, plent->uuid, sizeof(uuid));
 
  while ( plent != NULL ) {
   roar_vio_printf(vio,
-#ifdef HAVE_LIB_UUID
-                  "%s=%s=%s=%s=%s=%s=%s=%s=long:0x%.16LX/short:0x%.8X/uuid:%s=0x%.8x/%i=%s(0x%x)=%f\n",
-#else
-                  "%s=%s=%s=%s=%s=%s=%s=%s=long:0x%.16LX/short:0x%.8X=0x%.8x/%i=%s(0x%x)=%f\n",
-#endif
+                  "%s=%s=%s=%s=%s=%s=%s=%s=long:0x%.16" __ll "X/short:0x%.8lX/uuid:%s=0x%.8lx/%i=%s(0x%x)=%f\n",
                         roar_codec2str(plent->codec),
                         rpld_ple_time_hr(plent),
                         plent->meta.album,
@@ -71,12 +63,10 @@ int ff_rpld_pl_export(struct fformat_handle * handle, struct roar_vio_calls * vi
                         plent->meta.performer,
                         plent->meta.version,
                         plent->io.filename,
-                        plent->global_tracknum,
-                        plent->global_short_tracknum,
-#ifdef HAVE_LIB_UUID
+                        (__longlong unsigned int)plent->global_tracknum,
+                        (long unsigned int)plent->global_short_tracknum,
                         uuid,
-#endif
-                        plent->meta.discid,
+                        (long unsigned int)plent->meta.discid,
                         plent->meta.tracknum,
                         roar_meta_strgenre(plent->meta.genre),
                         plent->meta.genre,
