@@ -144,12 +144,10 @@ int ff_xspf_pl_export(struct fformat_handle * handle, struct roar_vio_calls * vi
    }
   }
 
-#ifdef HAVE_LIB_UUID
   strcpy(buf, "tantalos://");
-  uuid_unparse(cur->uuid, buf+11);
+  roar_uuid2str(buf+11, cur->uuid, sizeof(buf)-11);
   new_text_node(track, "location", buf); // as alternative location to the one above, if any.
   new_text_node(track, "identifier", buf);
-#endif
 
   if (cur->meta.album[0] != 0)
    new_text_node(track, "album", cur->meta.album);
@@ -254,11 +252,9 @@ static void ff_xspf_pl_import_track(struct rpld_playlist  * pl, xmlNode * track)
   if (!strcmp(name, "location")) {
    // TODO.
    if ( !strncmp(value, "tantalos://", 11) ) {
-#ifdef HAVE_LIB_UUID
     // For error checking see below.
     value += 11;
-    uuid_parse(value, plent->uuid);
-#endif
+    roar_str2uuid(plent->uuid, value);
    } else {
     // Very ugly workaround as libxml2 has bad docs.
     // Prey this will never explode.
@@ -267,15 +263,13 @@ static void ff_xspf_pl_import_track(struct rpld_playlist  * pl, xmlNode * track)
      plent->io.filename[sizeof(plent->io.filename)-1] = 0;
     }
    }
-#ifdef HAVE_LIB_UUID
   } else if (!strcmp(name, "identifier")) {
    if ( !strncmp(value, "tantalos://", 11) ) {
     value += 11;
     // We can ignore errors here as uuid_parse() does not alter the UUID if parsing failed.
     // This is the documented behavior. Hope nobody will break it somewhen...
-    uuid_parse(value, plent->uuid);
+    roar_str2uuid(plent->uuid, value);
    }
-#endif
   } else if (!strcmp(name, "title")) {
    dst = plent->meta.title;
    dst_len = sizeof(plent->meta.title);
