@@ -36,16 +36,23 @@ static struct {
  {RPLD_FF_PLF_M3U,   "m3u",   "m3u"   },
  {RPLD_FF_PLF_XSPF,  "xspf",  "xspf"  },
  {RPLD_FF_PLF_PLAIN, "plain", NULL    },
+ {RPLD_FF_PLF_URAS,  "uras",  "xml"   },
  {-1, NULL, NULL}
 };
 
 int          fformat_str2plf(const char * str) {
  int i;
 
+ if ( str == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
+  return -1;
+ }
+
  for (i = 0; ff_plf[i].id != -1; i++)
   if ( !strcasecmp(str, ff_plf[i].name) )
    return ff_plf[i].id;
 
+ roar_err_set(ROAR_ERROR_NOENT);
  return -1;
 }
 
@@ -56,6 +63,7 @@ const char * fformat_plf2str(int plf) {
   if ( ff_plf[i].id == plf )
    return ff_plf[i].name;
 
+ roar_err_set(ROAR_ERROR_NOENT);
  return NULL;
 }
 
@@ -74,6 +82,7 @@ static struct {
  {RPLD_FF_PLF_PLS,   NULL, NULL, ff_pls_pl_export,   NULL,               NULL},
 #ifdef HAVE_LIB_XML2
  {RPLD_FF_PLF_XSPF,  NULL, NULL, ff_xspf_pl_export,  ff_xspf_pl_import,  NULL},
+ {RPLD_FF_PLF_URAS,  NULL, NULL, ff_uras_pl_export,  ff_uras_pl_import,  NULL},
 #endif
  {-1, NULL, NULL, NULL, NULL, NULL}
 };
@@ -142,9 +151,11 @@ int                     fformat_playlist_import(struct fformat_handle * handle,
  if ( tmp_pl == NULL )
   return -1;
 
+ yield_watchdog_stop();
  yield_auto(RPLD_YIELD_START, RPLD_YIELD_TASK_PLAYBACK, RPLD_YIELD_TASK_NETWORK, RPLD_YIELD_NONBLOCK);
  ret = ff_handles[handle->table_id].pl_import(handle, vio, tmp_pl);
  yield_auto(RPLD_YIELD_STOP, RPLD_YIELD_TASK_PLAYBACK, RPLD_YIELD_TASK_NETWORK, RPLD_YIELD_NONBLOCK);
+ yield_watchdog_start();
 
  if ( ret == 0 ) {
   ret = rpld_pl_merge(pl, tmp_pl);
