@@ -33,6 +33,7 @@
 #define RPLD_FF_PLF_M3U        4
 #define RPLD_FF_PLF_XSPF       5
 #define RPLD_FF_PLF_PLAIN      6
+#define RPLD_FF_PLF_URAS       7
 
 #define RPLD_FF_PLF_DEFAULT    RPLD_FF_PLF_VCLT
 
@@ -71,6 +72,8 @@ int ff_pls_pl_export(struct fformat_handle * handle, struct roar_vio_calls * vio
 int ff_pls_pl_import(struct fformat_handle * handle, struct roar_vio_calls * vio, struct rpld_playlist  * pl);
 int ff_xspf_pl_export(struct fformat_handle * handle, struct roar_vio_calls * vio, struct rpld_playlist  * pl);
 int ff_xspf_pl_import(struct fformat_handle * handle, struct roar_vio_calls * vio, struct rpld_playlist  * pl);
+int ff_uras_pl_export(struct fformat_handle * handle, struct roar_vio_calls * vio, struct rpld_playlist  * pl);
+int ff_uras_pl_import(struct fformat_handle * handle, struct roar_vio_calls * vio, struct rpld_playlist  * pl);
 
 #endif
 
