@@ -26,7 +26,8 @@
 #ifndef _RPLD_NETWORK_H_
 #define _RPLD_NETWORK_H_
 
-#define RPLD_SOCKNAME_UNIX   "./.rpld"
+#define RPLD_SOCKNAME_UNIX   "~/.rpld"
+#define RPLD_SOCKNAME_UNIX_FALLBACK "/tmp/.rpld"
 #define RPLD_SOCKNAME_TCP    "localhost"
 #define RPLD_SOCKNAME_DECNET "::rpld"
 #define RPLD_SOCKNAME        RPLD_SOCKNAME_UNIX
