@@ -79,6 +79,8 @@ void playback_set_role(pli_t queue, const int role) {
  if ( plq == NULL )
   return;
 
+ roar_notify_core_emit_simple(RPLD_NOTIFY_PLAYBACK_ROLE, -1, queue, -1, role, -1, NULL, -1);
+
  plq->role = role;
 
  if ( !plq->playing ) {
@@ -114,6 +116,9 @@ static void _handle_queue_io(pli_t queue, struct rpld_queue * plq) {
  int is_playing;
 
  if ( (len = roar_vio_read(plq->io.fp, buf, sizeof(buf))) < 1 ) {
+  if ( len == -1 && roar_error == ROAR_ERROR_AGAIN )
+   return;
+
   is_playing = playback_is_playing(queue);
 
   playback_stop(queue, 0, 0);
@@ -139,7 +144,7 @@ static void _handle_queue_io(pli_t queue, struct rpld_queue * plq) {
 }
 
 void playback_check(int block) {
- struct roar_vio_select vioset[2*(MAX_PLAYLISTS/2)]; // we need 2 handles per queue
+ struct roar_vio_select vioset[3*(MAX_PLAYLISTS/2)]; // we need 3 handles per queue
                                                      // and guess we have a max of MAX_PLAYLISTS/2 queues.
  struct rpld_queue * plq;
  struct rpld_playlist * pl;
@@ -147,7 +152,7 @@ void playback_check(int block) {
  size_t i;
  size_t count = 0;
  int is_playing = 0;
- int found_active_stream = 0;
+ size_t left_handles = 0;
 
 #ifdef ROAR_TARGET_WIN32
  if ( block == RPLD_YIELD_NONBLOCK ) {
@@ -182,10 +187,36 @@ void playback_check(int block) {
   vioset[count].ud.vp = pl;
   count++;
 
+  left_handles += 2;
+
+  if ( plq->io.fp == &(plq->io.file) ) {
+   ROAR_VIO_SELECT_SETVIO(&(vioset[count]), &(plq->io.file), ROAR_VIO_SELECT_NONE|ROAR_VIO_SELECT_NO_RETEST);
+  } else {
+   ROAR_VIO_SELECT_SETVIO(&(vioset[count]), &(plq->io.file), ROAR_VIO_SELECT_READ);
+   left_handles++;
+  }
+  vioset[count].ud.vp = pl;
+  count++;
+
   is_playing = 1;
  }
 
  if ( is_playing ) {
+  tv.sec  = 0;
+  tv.nsec = 32*1000;
+
+  yield_watchdog_trigger();
+  if ( roar_vio_select(vioset, count, &tv, NULL) == -1 )
+   return;
+  yield_watchdog_trigger();
+
+  for (i = 0; i < count; i++) {
+   if ( vioset[i].eventsa & vioset[i].eventsq ) {
+    vioset[i].eventsq |= ROAR_VIO_SELECT_NO_RETEST;
+    left_handles--;
+   }
+  }
+
   tv.sec  = 0;
 
   if ( block ) {
@@ -196,31 +227,25 @@ void playback_check(int block) {
 
   ROAR_DBG("playback_check(void) = ?");
 
-  while (!found_active_stream) {
-   ROAR_DBG("playback_check(void) = ?");
-   // if we have less than one handle we can not do anything anyway so we just return.
-   if ( roar_vio_select(vioset, count, &tv, NULL) < 1 )
-    return;
+  ROAR_DBG("playback_check(void) = ?");
+  yield_watchdog_trigger();
+  // if we have less than one handle we can not do anything anyway so we just return.
+  ROAR_DBG("playback_check(void) = ?");
+  if ( left_handles )
+   if ( roar_vio_select(vioset, count, &tv, NULL) == -1 )
+     return;
+  ROAR_DBG("playback_check(void) = ?");
+  yield_watchdog_trigger();
 
-   for (i = 0; i < count; i += 2) {
-    if ( !(vioset[i+0].eventsa & ROAR_VIO_SELECT_WRITE) )
-     continue;
-    if ( !(vioset[i+1].eventsa & ROAR_VIO_SELECT_READ) )
-     continue;
-    found_active_stream++;
-    pl = vioset[i].ud.vp;
-    _handle_queue_io(pl->id, pl->queue);
-   }
-
-   ROAR_DBG("playback_check(void): found_active_stream=%i", found_active_stream);
-
-   if ( block ) {
-    if ( !found_active_stream )
-     roar_usleep(25000);
-   } else {
-    found_active_stream = 1; // terminate loop.
-   }
+  for (i = 0; i < count; i += 3) {
+   if ( !(vioset[i+0].eventsa & ROAR_VIO_SELECT_WRITE) )
+    continue;
+   if ( !(vioset[i+1].eventsa & ROAR_VIO_SELECT_READ) && !(vioset[i+2].eventsa & ROAR_VIO_SELECT_READ) )
+    continue;
+   pl = vioset[i].ud.vp;
+   _handle_queue_io(pl->id, pl->queue);
   }
+
   ROAR_DBG("playback_check(void) = ?");
  } else {
   ROAR_DBG("playback_check(void) = ?");
@@ -403,6 +428,8 @@ struct rpld_playlist_pointer * playback_get_pointer(pli_t queue, int pointer) {
 void playback_play_all(void) {
  size_t i;
 
+ roar_notify_core_emit_simple(RPLD_NOTIFY_PLAYBACK_PLAY, -1, -1, -1, -1, -1, NULL, -1);
+
  for (i = 0; i < MAX_PLAYLISTS; i++) {
   if ( g_playlists[i] == NULL )
    continue;
@@ -417,6 +444,8 @@ void playback_play_all(void) {
 void playback_stop_all(size_t offset, int kick) {
  size_t i;
 
+ roar_notify_core_emit_simple(RPLD_NOTIFY_PLAYBACK_STOP, -1, -1, -1, offset, kick, NULL, -1);
+
  for (i = 0; i < MAX_PLAYLISTS; i++) {
   if ( g_playlists[i] == NULL )
    continue;
@@ -452,9 +481,7 @@ int  playback_play(pli_t queue) {
  int codec;
  ssize_t magic_len = 0;
  char magic[64];
-#ifdef HAVE_LIB_UUID
  char uuid[37];
-#endif
  const char * content_type;
  struct rpld_queue * plq;
  int err;
@@ -474,6 +501,11 @@ int  playback_play(pli_t queue) {
  if ( plq->playing )
   return 0;
 
+ if ( !(plq->backend->flags & BACKEND_FLAG_CONNECTED) ) {
+  roar_err_set(ROAR_ERROR_NOTCONN);
+  return -1;
+ }
+
  ROAR_DBG("playback_play(void) = ?");
 
  if ( (plent = rpld_pl_get_first(pl)) == NULL ) {
@@ -487,14 +519,9 @@ int  playback_play(pli_t queue) {
  ROAR_DBG("playback_play(void) = ?");
 
  if ( *(plent->io.filename) == 0 ) {
-#ifdef HAVE_LIB_UUID
-  uuid_unparse(plent->uuid, uuid);
+  roar_uuid2str(uuid, plent->uuid, sizeof(uuid));
   snprintf(plent->io.filename, sizeof(plent->io.filename), "tantalos://%s", uuid);
   plent->io.filename[sizeof(plent->io.filename)-1] = 0;
-#else
-  roar_err_set(ROAR_ERROR_NOTSUP);
-  return -1;
-#endif
  }
 
  ROAR_DBG("playback_play(void) = ?");
@@ -623,6 +650,16 @@ int  playback_play(pli_t queue) {
 
  plq->playing = 1;
 
+ roar_notify_core_emit_simple(RPLD_NOTIFY_PLAYBACK_PLAY, -1, queue, -1, -1, -1, NULL, -1);
+
+ // set rpg data:
+ // only set in case non-default parameters are requested.
+ if ( plent->rpg.mode != ROAR_RPGMODE_DEFAULT ) {
+  if ( roar_stream_set_rpg(&(plq->backend->con), &(plq->stream), &(plent->rpg)) == -1 ) {
+   ROAR_WARN("playback_play(queue=%u): Can not set RPG settings: %s", (unsigned)queue, roar_errorstring);
+  }
+ }
+
  // set role for new stream:
  playback_set_role(queue, plq->role);
 
@@ -650,11 +687,16 @@ int  playback_next(pli_t queue) {
   return -1;
  }
 
+ roar_notify_core_emit_simple(RPLD_NOTIFY_PLAYBACK_NEXT, -1, queue, -1, -1, -1, NULL, -1);
+
  was_playing = plq->playing;
  if ( playback_stop(queue, 0, 1) == -1 )
   return -1;
 
- old = rpld_pl_shift(pl);
+ // get next element, if there is none try autoqueue.
+ if ( (old = rpld_pl_shift(pl)) == NULL )
+  if ( autoqueue(queue) != -1 )
+   old = rpld_pl_shift(pl);
 
  if ( old != NULL ) {
   rpld_pl_add(plq->history, old, -1);
@@ -690,6 +732,8 @@ int  playback_prev(pli_t queue) {
   return -1;
  }
 
+ roar_notify_core_emit_simple(RPLD_NOTIFY_PLAYBACK_PREV, -1, queue, -1, -1, -1, NULL, -1);
+
  was_playing = plq->playing;
 
  old = rpld_pl_shift(plq->history);
@@ -722,6 +766,8 @@ int  playback_stop(pli_t queue, size_t offset, int kick) {
   return 0;
  }
 
+ roar_notify_core_emit_simple(RPLD_NOTIFY_PLAYBACK_STOP, -1, queue, -1, offset, kick, NULL, -1);
+
  // first we kick, than we close the IO handles to avoid race conditions.
  roar_kick(&(plq->backend->con),
            ROAR_OT_STREAM,
@@ -803,6 +849,8 @@ int  playback_pause(pli_t queue, int how) {
 #endif
 
  rpld_pl_unref(pl);
+
+ roar_notify_core_emit_simple(RPLD_NOTIFY_PLAYBACK_PAUSE, -1, queue, -1, how, -1, NULL, -1);
 }
 
 int  playback_is_playing(pli_t queue) {
@@ -927,6 +975,8 @@ int  playback_set_volume_mu16(pli_t queue, uint16_t mono) {
 
  plq->volume_mono = mono;
 
+ roar_notify_core_emit_simple(RPLD_NOTIFY_PLAYBACK_VOLUME, -1, queue, -1, mono, -1, NULL, -1);
+
  // if we are not playing we are done
  if ( !plq->playing ) {
   rpld_pl_unref(pl);
@@ -993,14 +1043,21 @@ int      playback_get_volume_mpc(pli_t queue) {
  return playback_get_volume_mu16(queue) / 655;
 }
 
-struct roar_stream * playback_get_stream (pli_t queue) {
+const struct roar_stream * playback_get_stream (pli_t queue) {
  struct rpld_playlist * pl;
  struct rpld_queue * plq = _get_queue(queue, &pl);
  static struct roar_stream stream;
 
+ // TODO: FIXME: Why don't we just returen the stream object from the plq?
+
  if ( plq == NULL )
   return NULL;
 
+ if ( !plq->playing ) {
+  rpld_pl_unref(pl);
+  return NULL;
+ }
+
  if ( roar_get_stream(&(plq->backend->con),
                       &stream,
                       roar_stream_get_id(&(plq->stream))) == -1 ) {
