@@ -1,4 +1,4 @@
-//playlist.c:
+     //playlist.c:
 
 /*
  *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2012
@@ -24,6 +24,7 @@
  */
 
 #include "rpld.h"
+#include <ctype.h>
 
 struct rpld_playlist * g_playlists[MAX_PLAYLISTS];
 
@@ -45,14 +46,22 @@ struct rpld_playlist_entry * rpld_ple_new     (void) {
  plent->global_short_tracknum |= (counter & 0xFFFF) << 8;
  plent->global_short_tracknum |= rand_data ^ RPLD_PL_MAGIC;
 
+ plent->start                  = -1;
+ plent->end                    = -1;
+ plent->rpg.mode               = ROAR_RPGMODE_DEFAULT;
+ plent->rpg.mul                =  1;
+ plent->rpg.div                =  1;
  plent->codec                  = -1;
  plent->meta.genre             = -1;
 
+ plent->meta.extra.kv          = NULL;
+ plent->meta.extra.kvlen       = -1;
+ plent->meta.extra.kv_store    = NULL;
+ plent->meta.extra.kv_storelen = -1;
+
  plent->likeness               =  1;
 
-#ifdef HAVE_LIB_UUID
- uuid_generate_random(plent->uuid);
-#endif
+ roar_uuid_gen_random(plent->uuid);
 
  counter++;
 
@@ -63,6 +72,15 @@ void                         rpld_ple_free    (struct rpld_playlist_entry * plen
  if ( plent == NULL )
   return;
 
+ if ( plent->meta.extra.kv_store != NULL )
+  roar_mm_free(plent->meta.extra.kv_store);
+
+ if ( plent->meta.extra.kv != NULL )
+  roar_mm_free(plent->meta.extra.kv);
+
+ memset(plent, 0, sizeof(*plent));
+ strncpy(plent->meta.title, "<<<freed>>>", sizeof(plent->meta.title));
+
  roar_mm_free(plent);
 }
 
@@ -82,10 +100,16 @@ struct rpld_playlist_entry * rpld_ple_copy    (struct rpld_playlist_entry * plen
 
  memcpy(ret, plent, sizeof(struct rpld_playlist_entry));
  memset(&(ret->list), 0, sizeof(ret->list));
+ memset(&(ret->meta.extra), 0, sizeof(ret->meta.extra));
 
  ret->global_tracknum       = trknl;
  ret->global_short_tracknum = trkns;
 
+ ret->meta.extra.kv_store = roar_keyval_copy(&(ret->meta.extra.kv), plent->meta.extra.kv, plent->meta.extra.kvlen);
+ ret->meta.extra.kv_storelen = -1;
+ if ( ret->meta.extra.kv_store != NULL )
+  ret->meta.extra.kvlen = plent->meta.extra.kvlen;
+
  return ret;
 }
 
@@ -155,6 +179,182 @@ int                          rpld_ple_set_time_hr (struct rpld_playlist_entry *
  return 0;
 }
 
+
+static inline int __iscasein(const char * haystack, const char * needle) {
+ size_t i, j;
+ int lower_first = tolower(*needle);
+
+ // consider the following magic.
+ for (i = 0; haystack[i]; i++) {
+  if ( tolower(haystack[i]) == lower_first ) {
+   for (j = 0; haystack[i+j] && needle[j]; j++)
+    if ( tolower(haystack[i+j]) != tolower(needle[j]) )
+     break;
+   if ( !needle[j] )
+    return 1;
+  }
+ }
+
+ return 0;
+}
+
+int                          rpld_ple_filter  (const struct rpld_playlist_entry * plent, const void * needle, const char * key, int options) {
+ const char * barn[8];
+ const char ** haystacks;
+ ssize_t haystackcount = 0;
+ ssize_t i, j;
+ ssize_t needlelen;
+ int ret = 0;
+ union {
+  const ei_t * ei;
+  const ei_short_t * ei_short;
+  const roar_uuid_t * uuid;
+  const discid_t * discid;
+  const int  * si;
+  const char * str;
+  const void * vp;
+ } subject;
+
+ subject.vp = needle;
+
+ // a key of NULL is valid and means 'search in all keys'.
+ if ( plent == NULL || needle == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
+  return -1;
+ }
+
+ // if no sources are given this doesn't match.
+ if ( !(options & (RPLD_PLE_FILTER_NORMKEY|RPLD_PLE_FILTER_EXTKEY)) )
+  return 0;
+
+ if ( options & RPLD_PLE_FILTER_BNEEDLE ) {
+  // we can not do a binary search on non-binary data.
+  if ( options & RPLD_PLE_FILTER_EXTKEY ) {
+   roar_err_set(ROAR_ERROR_INVAL);
+   return -1;
+  }
+
+  // key needs to be given as we use it as type in binary mode.
+  if ( key == NULL ) {
+   roar_err_set(ROAR_ERROR_FAULT);
+   return -1;
+  }
+
+  if ( !strcasecmp(key, "long") ) {
+   return *subject.ei == plent->global_tracknum ? 1 : 0;
+  } else if ( !strcasecmp(key, "short") ) {
+   return *subject.ei_short == plent->global_short_tracknum ? 1 : 0;
+  } else if ( !strcasecmp(key, "uuid") ) {
+   return roar_uuid_eq(*subject.uuid, plent->uuid);
+  } else if ( !strcasecmp(key, "discid") ) {
+   return *subject.discid == plent->meta.discid ? 1 : 0;
+  } else if ( !strcasecmp(key, "genre") ) {
+   return *subject.si == plent->meta.genre ? 1 : 0;
+  } else if ( !strcasecmp(key, "tracknum") || !strcasecmp(key, "tracknumber") ) {
+   return *subject.si == plent->meta.tracknum ? 1 : 0;
+  } else {
+   roar_err_set(ROAR_ERROR_NOTSUP);
+   return -1;
+  }
+ }
+
+ // first count how many keys we need to search:
+ if ( options & RPLD_PLE_FILTER_NORMKEY ) {
+#define _ck(keyname) if ( key == NULL || !strcasecmp(key, (keyname)) ) { haystackcount++; }
+  _ck("filename")
+  _ck("album")
+  _ck("title")
+  _ck("artist")
+  _ck("performer")
+  _ck("version")
+#undef _ck
+ }
+
+ if ( (options & RPLD_PLE_FILTER_EXTKEY) && plent->meta.extra.kv != NULL )
+  for (i = 0; i < plent->meta.extra.kvlen; i++)
+   if ( plent->meta.extra.kv[i].key != NULL && (key == NULL || !strcasecmp(plent->meta.extra.kv[i].key, key)) )
+    haystackcount++;
+
+ // now allocate space for all the hay:
+ if ( haystackcount <= (ssize_t)(sizeof(barn)/sizeof(*barn)) ) {
+  haystacks = barn;
+ } else {
+  haystacks = roar_mm_malloc(haystackcount*sizeof(*haystacks));
+  if ( haystacks == NULL )
+   return -1;
+ }
+
+ // Build the list of haystacks:
+ j = 0;
+
+ if ( options & RPLD_PLE_FILTER_NORMKEY ) {
+#define _ck(keyname,member) if ( key == NULL || !strcasecmp(key, (keyname)) ) { haystacks[j++] = plent->member; }
+  _ck("filename", io.filename)
+  _ck("album", meta.album)
+  _ck("title", meta.title)
+  _ck("artist", meta.artist)
+  _ck("performer", meta.performer)
+  _ck("version", meta.version)
+#undef _ck
+ }
+
+ if ( (options & RPLD_PLE_FILTER_EXTKEY) && plent->meta.extra.kv != NULL )
+  for (i = 0; i < plent->meta.extra.kvlen; i++)
+   if ( plent->meta.extra.kv[i].key != NULL && (key == NULL || !strcasecmp(plent->meta.extra.kv[i].key, key)) )
+    haystacks[j++] = plent->meta.extra.kv[i].value;
+
+ // now do the search:
+ // TODO: this should be optimized.
+ needlelen = roar_mm_strlen(subject.str);
+ for (i = 0; !ret && i < haystackcount; i++) {
+  if ( haystacks[i] == NULL )
+   continue;
+
+  if ( (options & RPLD_PLE_FILTER_ANCHORB) & (options & RPLD_PLE_FILTER_ANCHORE) ) {
+   if ( options & RPLD_PLE_FILTER_CASE ) {
+    if ( !strcmp(haystacks[i], subject.str) )
+     ret = 1;
+   } else {
+    if ( !strcasecmp(haystacks[i], subject.str) )
+     ret = 1;
+   }
+  } else if ( options & RPLD_PLE_FILTER_ANCHORB ) {
+   if ( options & RPLD_PLE_FILTER_CASE ) {
+    if ( !strncmp(haystacks[i], subject.str, needlelen) )
+     ret = 1;
+   } else {
+    if ( !strncasecmp(haystacks[i], subject.str, needlelen) )
+     ret = 1;
+   }
+  } else if ( options & RPLD_PLE_FILTER_ANCHORE ) {
+   j = roar_mm_strlen(haystacks[i]);
+   if ( needlelen > j )
+    continue;
+
+   if ( options & RPLD_PLE_FILTER_CASE ) {
+    if ( !strcmp(haystacks[i] + j - needlelen, subject.str) )
+     ret = 1;
+   } else {
+    if ( !strcasecmp(haystacks[i] + j - needlelen, subject.str) )
+     ret = 1;
+   }
+  } else {
+   if ( options & RPLD_PLE_FILTER_CASE ) {
+    if ( strstr(haystacks[i], subject.str) != NULL )
+     ret = 1;
+   } else {
+    ret = __iscasein(haystacks[i], subject.str);
+   }
+  }
+ }
+
+ // Free space space we alloced because barn was too small.
+ if ( haystacks != barn )
+  roar_mm_free(haystacks);
+
+ return ret;
+}
+
 struct rpld_playlist       * rpld_pl_new      (void) {
  struct rpld_playlist * pl = roar_mm_malloc(sizeof(struct rpld_playlist));
 
@@ -307,6 +507,8 @@ struct rpld_playlist_entry * rpld_pl_shift    (struct rpld_playlist * pl) {
   pl->likeness_hint -= first->likeness;
   pl->size_hint--;
 
+  memset(&(first->list), 0, sizeof(first->list));
+
   return first;
  }
 }
@@ -458,13 +660,11 @@ struct rpld_playlist_entry * rpld_pl_search   (struct rpld_playlist        * pl,
       _match();
      }
     break;
-#ifdef HAVE_LIB_UUID
     case RPLD_PL_ST_UUID:
-     if ( !uuid_compare(cur->uuid, search->subject.uuid) ) {
+     if ( roar_uuid_eq(cur->uuid, search->subject.uuid) ) {
       _match();
      }
     break;
-#endif
    case RPLD_PL_ST_NUM:
    case RPLD_PL_ST_RANDOM:
      if ( counter == target_num ) {
@@ -868,7 +1068,7 @@ struct rpld_playlist       * rpld_pl_get_by_name(const char * name) {
 }
 
 size_t                       rpld_pl_num      (struct rpld_playlist * pl) {
- struct rpld_playlist_entry * cur;
+ struct rpld_playlist_entry * cur, * next;
  size_t count = 0;
 
  if ( pl == NULL )
@@ -878,7 +1078,13 @@ size_t                       rpld_pl_num      (struct rpld_playlist * pl) {
 
  while ( cur != NULL ) {
   count++;
-  cur = cur->list.next;
+  next = cur->list.next;
+  if ( next == cur ) {
+   ROAR_ERR("rpld_pl_num(pl=%p): Application error: PLE %p points to itself as next.", pl, cur);
+   roar_panic(ROAR_FATAL_ERROR_MEMORY_CORRUPTION, NULL);
+   return count;
+  }
+  cur = next;
  }
 
  return count;
