@@ -38,6 +38,16 @@ typedef uint64_t     ei_t;
 typedef uint32_t     ei_short_t;
 typedef uint32_t     discid_t;
 
+// PLE filter options:
+#define RPLD_PLE_FILTER_NONE     0x0000
+#define RPLD_PLE_FILTER_NORMKEY  0x0001 /* search in normal keys             */
+#define RPLD_PLE_FILTER_EXTKEY   0x0002 /* search in extra keys              */
+#define RPLD_PLE_FILTER_CASE     0x0004 /* search case sensetive             */
+#define RPLD_PLE_FILTER_ANCHORB  0x0008 /* "begin" anchor: ^                 */
+#define RPLD_PLE_FILTER_ANCHORE  0x0010 /* "end" anchor: $                   */
+#define RPLD_PLE_FILTER_BNEEDLE  0x0020 /* needle is binary (uuid, GTN, ...) */
+
+// ???:
 #define RPLD_PL_FLAG_NONE        0x0000
 #define RPLD_PL_FLAG_VIRTUAL     0x0001
 #define RPLD_PL_FLAG_UNLINKED    0x0002
@@ -69,6 +79,8 @@ typedef uint32_t     discid_t;
 #define RPLD_PL_SO_NONE             0x0000
 #define RPLD_PL_SO_PREVIOUS         0x0001
 
+typedef int_least32_t rpld_mus_t;
+
 struct rpld_playlist_entry {
 /*
   trancknum:
@@ -88,10 +100,11 @@ struct rpld_playlist_entry {
  */
  ei_t       global_tracknum;
  ei_short_t global_short_tracknum;
-#ifdef HAVE_LIB_UUID
- uuid_t     uuid;
-#endif
+ roar_uuid_t uuid;
  time_t length;
+ rpld_mus_t start; // start time (within the resource) in ms.
+ rpld_mus_t end;   // end time (within the resource) in ms.
+ struct roar_stream_rpg rpg;
  int codec;
  struct {
   char filename[RPLD_MAX_PLF_LEN];
@@ -105,6 +118,12 @@ struct rpld_playlist_entry {
   int      genre;
   int      tracknum;
   discid_t discid;
+  struct {
+   struct roar_keyval * kv;
+   ssize_t kvlen;
+   void * kv_store;
+   ssize_t kv_storelen;
+  } extra;
  } meta;
  struct {
   struct rpld_playlist_entry * next;
@@ -133,9 +152,7 @@ struct rpld_playlist_search {
  union {
   ei_t        long_tracknum;
   ei_short_t short_tracknum;
-#ifdef HAVE_LIB_UUID
-  uuid_t               uuid;
-#endif
+  roar_uuid_t          uuid;
   size_t                num;
   float            likeness;
   pli_t                  pl;
@@ -163,6 +180,9 @@ struct rpld_playlist_entry * rpld_ple_cut_out_next (struct rpld_playlist_entry *
 const char                 * rpld_ple_time_hr (const struct rpld_playlist_entry * plent);
 int                          rpld_ple_set_time_hr (struct rpld_playlist_entry * plent, char * time);
 
+//  SEARCHPLE {"search string"|0xdiscid|uuid} [{AS|IN} {ALL|TITLE|ARTIST|VERSION|FILENAME|DISCID|UUID|TAG:"Tagname"}] [FROM {"Name"|ID|ANY}]
+int                          rpld_ple_filter  (const struct rpld_playlist_entry * plent, const void * needle, const char * key, int options);
+
 struct rpld_playlist       * rpld_pl_new      (void);
 int                          rpld_pl_ref      (struct rpld_playlist * pl);
 int                          rpld_pl_unref    (struct rpld_playlist * pl);
