@@ -133,6 +133,8 @@ static int                         _ple_set_meta    (struct rpld_playlist_entry
  return 0;
 }
 static const char *                 _ple_get_meta    (struct rpld_playlist_entry * plent, const char * key) {
+ const struct roar_keyval * kv;
+
  __ple_check(NULL)
 
  if ( !strcasecmp(key, "album") )
@@ -146,9 +148,54 @@ static const char *                 _ple_get_meta    (struct rpld_playlist_entry
  if ( !strcasecmp(key, "version") )
   return plent->meta.version;
 
+ if ( plent->meta.extra.kv != NULL ) {
+  kv = roar_keyval_lookup(plent->meta.extra.kv, key, plent->meta.extra.kvlen, 0);
+  if ( kv != NULL )
+   return kv->value;
+ }
+
  roar_err_set(ROAR_ERROR_NOENT);
  return NULL;
 }
+static int              _ple_set_meta_int(struct rpld_playlist_entry * plent, const char * key, rpld_maxint_t val) {
+ __ple_check(-1)
+
+ if ( !strcasecmp(key, "length") ) {
+  plent->length = val;
+ } else if ( !strcasecmp(key, "codec") ) {
+  plent->codec = val;
+ } else if ( !strcasecmp(key, "genre") ) {
+  plent->meta.genre = val;
+ } else if ( !strcasecmp(key, "tracknum") || !strcasecmp(key, "tracknumber") ) {
+  plent->meta.tracknum = val;
+ } else if ( !strcasecmp(key, "discid") ) {
+  plent->meta.discid = val;
+ } else {
+  roar_err_set(ROAR_ERROR_NOTSUP);
+  return -1;
+ }
+
+ return 0;
+}
+
+static rpld_maxint_t    _ple_get_meta_int(struct rpld_playlist_entry * plent, const char * key) {
+ __ple_check((rpld_maxint_t)-1)
+
+ if ( !strcasecmp(key, "length") )
+  return plent->length;
+ if ( !strcasecmp(key, "codec") )
+  return plent->codec;
+ if ( !strcasecmp(key, "genre") )
+  return plent->meta.genre;
+ if ( !strcasecmp(key, "tracknum") || !strcasecmp(key, "tracknumber") )
+  return plent->meta.tracknum;
+ if ( !strcasecmp(key, "discid") )
+  return (uint32_t)plent->meta.discid;
+
+ roar_err_set(ROAR_ERROR_NOENT);
+ return (rpld_maxint_t)-1;
+}
+
 static int                          _ple_get_codec   (struct rpld_playlist_entry * plent) {
  __ple_check(-1)
  return plent->codec;
@@ -163,35 +210,23 @@ static int                          _ple_set_uuid    (struct rpld_playlist_entry
   return -1;
  }
 
-#ifdef HAVE_LIB_UUID
- if ( uuid_parse(uuid, plent->uuid) == 0 )
+ if ( roar_str2uuid(plent->uuid, uuid) == 0 )
   return 0;
 
  roar_err_set(ROAR_ERROR_ILLSEQ);
  return -1;
-#else
- roar_err_set(ROAR_ERROR_NOTSUP);
- return -1;
-#endif
 }
 static const char *                 _ple_get_uuid    (struct rpld_playlist_entry * plent, char * buf) {
-#ifdef HAVE_LIB_UUID
  static char buffer[37];
-#endif
 
  __ple_check(NULL)
 
-#ifdef HAVE_LIB_UUID
  if ( buf == NULL )
   buf = buffer;
 
- uuid_unparse(plent->uuid, buf);
+ roar_uuid2str(buf, plent->uuid, sizeof(buffer));
 
  return buf;
-#else
- roar_err_set(ROAR_ERROR_NOTSUP);
- return NULL;
-#endif
 }
 
 static int                          _ple_set_likeness(struct rpld_playlist_entry * plent, float likeness) {
@@ -213,6 +248,24 @@ static struct rpld_playlist_entry * _ple_get_next    (struct rpld_playlist_entry
  return plent->list.next;
 }
 
+static int     _ple_set_offset(struct rpld_playlist_entry * plent, int_least32_t   start, int_least32_t   end) {
+ __ple_check(-1)
+
+ plent->start = start;
+ plent->end   = end;
+
+ return 0;
+}
+static int     _ple_get_offset(struct rpld_playlist_entry * plent, int_least32_t * start, int_least32_t * end) {
+ __ple_check(-1)
+ if ( start != NULL )
+  *start = plent->start;
+ if ( end != NULL )
+  *end = plent->end;
+
+ return 0;
+}
+
 #define _begin(ret) \
  struct rpld_playlist * playlist = rpld_pl_get_by_id(pl); \
  int err; \
@@ -427,12 +480,12 @@ static ssize_t _history_get_size(rpld_pli_t history) {
  return ret;
 }
 
-static struct rpld_playlist_pointer * _plp_new     (rpld_pli_t playlist, const char * type, void * argp) {
+static struct rpld_playlist_pointer * _plp_new     (rpld_pli_t playlist, const char * type, const void * argp) {
  struct rpld_playlist_search pls;
  union {
-  void * vp;
-  size_t * num;
-  float * likeness;
+  const void * vp;
+  const size_t * num;
+  const float * likeness;
   const char * uuid;
  } arg = {.vp = argp};
  struct rpld_playlist * pl;
@@ -448,10 +501,8 @@ static struct rpld_playlist_pointer * _plp_new     (rpld_pli_t playlist, const c
 
  if ( !strcasecmp(type, "num") ) {
   pls.type = RPLD_PL_ST_NUM;
-#ifdef HAVE_LIB_UUID
  } else if ( !strcasecmp(type, "uuid") ) {
   pls.type = RPLD_PL_ST_UUID;
-#endif
  } else if ( !strcasecmp(type, "likeness") ) {
   pls.type = RPLD_PL_ST_LIKENESS;
  } else if ( !strcasecmp(type, "random") ) {
@@ -482,14 +533,12 @@ static struct rpld_playlist_pointer * _plp_new     (rpld_pli_t playlist, const c
  }
 
  switch (pls.type) {
-#ifdef HAVE_LIB_UUID
   case RPLD_PL_ST_UUID:
-    if ( uuid_parse(arg.uuid, pls.subject.uuid) != 0 ) {
+    if ( roar_str2uuid(pls.subject.uuid, arg.uuid) != 0 ) {
      roar_err_set(ROAR_ERROR_ILLSEQ);
      return NULL;
     }
    break;
-#endif
   case RPLD_PL_ST_NUM:
     pls.subject.num = *arg.num;
    break;
@@ -580,6 +629,23 @@ static int   _playback_set_volume(rpld_pli_t queue, float volume) {
  return playback_set_volume_mu16(queue, (uint16_t)volume);
 }
 
+struct roar_connection * _backend_get_con(struct rpld_backend * backend) {
+ if ( backend == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
+  return NULL;
+ }
+
+ if ( !(backend->flags & BACKEND_FLAG_CONNECTED) ) {
+  roar_err_set(ROAR_ERROR_NOTCONN);
+  return NULL;
+ }
+
+ if ( roar_connectionref(&(backend->con)) == -1 )
+  return NULL;
+
+ return &(backend->con);
+}
+
 static int            _fformat_playlist_export(rpld_fformat_t handle,
                                     struct roar_vio_calls * vio,
                                     rpld_pli_t pl) {
@@ -646,7 +712,8 @@ static struct __rpldph_binargv __binargv = {
    ._error2str = roar_error2str,
   },
   .control = {
-   ._exit = rpld_exit
+   ._exit = rpld_exit,
+   ._serverinfo = rpld_serverinfo_get
   },
   .ple = {
    ._new = rpld_ple_new,
@@ -655,6 +722,8 @@ static struct __rpldph_binargv __binargv = {
    ._set_res = _ple_set_res,
    ._set_meta = _ple_set_meta,
    ._get_meta = _ple_get_meta,
+   ._set_meta_int = _ple_set_meta_int,
+   ._get_meta_int = _ple_get_meta_int,
    ._get_codec = _ple_get_codec,
    ._get_length = _ple_get_length,
    ._set_uuid = _ple_set_uuid,
@@ -662,7 +731,9 @@ static struct __rpldph_binargv __binargv = {
    ._set_likeness = _ple_set_likeness,
    ._inc_likeness = _ple_inc_likeness,
    ._get_likeness = _ple_get_likeness,
-   ._get_next = _ple_get_next
+   ._get_next = _ple_get_next,
+   ._set_offset = _ple_set_offset,
+   ._get_offset = _ple_get_offset
   },
   .playlist = {
    ._new = _playlist_new,
@@ -708,7 +779,8 @@ static struct __rpldph_binargv __binargv = {
    ._set_volume = _playback_set_volume,
 
    ._get_ple = playback_get_ple,
-   ._get_mduc = playback_get_mduc
+   ._get_mduc = playback_get_mduc,
+   ._get_stream = playback_get_stream
   },
   .store = {
    ._save = store,
@@ -721,17 +793,22 @@ static struct __rpldph_binargv __binargv = {
    ._unref = backend_unref,
 
    ._get_name = backend_get_name,
+   ._get_con = _backend_get_con,
 
    ._blacklist_codec = backend_blacklist_codec,
+   ._blacklist_codec_global = backend_blacklist_codec_global,
    ._blacklist_check_codec = backend_blacklist_check_codec
   },
   .client = {
+   ._set_name = client_set_name,
+   ._get_name = client_get_name,
    ._set_playlist = client_set_playlist,
    ._get_playlist = client_get_playlist,
    ._set_queue = client_set_queue,
    ._get_queue = client_get_queue,
 
    ._str2proto = client_str2proto,
+   ._proto2str = client_proto2str
   },
   .fformat = {
    ._str2plf = fformat_str2plf,
@@ -756,7 +833,8 @@ static struct __rpldph_binargv __binargv = {
   .yield = {
    ._ay_trigger = _yield_ay_trigger,
    ._ay_pause   = _yield_ay_pause,
-   ._ay_unpause = _yield_ay_unpause
+   ._ay_unpause = _yield_ay_unpause,
+   ._watchdog_trigger = yield_watchdog_trigger
   }
  }
 };
