@@ -188,7 +188,7 @@ Id: 0
  _pks("file", ple->io.filename);
 
  if ( ple->length )
-  _pk("Time", "%llu", (long long unsigned int)ple->length);
+  _pk("Time", "%lu", (long unsigned int)ple->length);
 
  if ( ple->meta.album[0] != 0 )
   _pks("Album", ple->meta.album);
@@ -205,19 +205,19 @@ Id: 0
   _pk("Track", "%i", ple->meta.tracknum);
 
  if ( pos != -1 )
-  _pk("Pos", "%llu", (long long unsigned int)pos);
+  _pk("Pos", "%lu", (long unsigned int)pos);
 
  songid = ple->global_short_tracknum & (uint32_t)0x7FFFFFFF;
 
- _pk("Id", "%llu", (long long unsigned int)songid);
+ _pk("Id", "%lu", (long unsigned int)songid);
 
  return 0;
 }
 
-int proto_mpd_client_set_proto(int id, struct roar_vio_calls * vio, struct roar_buffer ** obuffer, void ** userdata) {
+int proto_mpd_client_set_proto(int id, struct roar_vio_calls * vio, struct roar_buffer ** obuffer, void ** userdata, const struct roar_keyval * para, ssize_t paralen, struct roar_dl_librarypara * pluginpara) {
 #define NEW_CLIENT_STRING "OK MPD " RPLD_PROTO_MPD_VERSION "\n"
 
- (void)id, (void)obuffer, (void)userdata;
+ (void)id, (void)obuffer, (void)userdata, (void)para, (void)paralen, (void)pluginpara;
 
  if ( roar_vio_puts(vio, NEW_CLIENT_STRING) != (sizeof(NEW_CLIENT_STRING)-1) )
   return -1;
@@ -225,14 +225,14 @@ int proto_mpd_client_set_proto(int id, struct roar_vio_calls * vio, struct roar_
  return 0;
 }
 
-int proto_mpd_client_handle(int id, struct roar_vio_calls * vio, struct roar_buffer ** obuffer, void ** userdata) {
+int proto_mpd_client_handle(int id, struct roar_vio_calls * vio, struct roar_buffer ** obuffer, void ** userdata, const struct roar_keyval * para, ssize_t paralen, struct roar_dl_librarypara * pluginpara) {
  struct rpld_proto_mpd_error error;
  ssize_t len;
  char * args;
  int    ret;
  char buffer[1024];
 
- (void)obuffer, (void)userdata;
+ (void)obuffer, (void)userdata, (void)para, (void)paralen, (void)pluginpara;
 
  if ( vio == NULL )
   return -1;
@@ -289,7 +289,7 @@ int proto_mpd_pcmd(int id, struct roar_vio_calls * vio, struct rpld_proto_mpd_er
  pli_t queue = client_get_queue(id);
  struct rpld_playlist * pl;
  struct rpld_playlist_entry * plent;
- struct roar_stream         * s;
+ const struct roar_stream   * s;
  ssize_t pos = 0;
  int i;
 
@@ -337,7 +337,7 @@ db_update: 1264178670
    if ( (plent = playback_get_ple(queue)) != NULL && (s = playback_get_stream(queue)) != NULL ) {
     roar_vio_printf(vio, "song: 0\n");
     roar_vio_printf(vio, "songid: %u\n", plent->global_short_tracknum & (uint32_t)0x7FFFFFFF);
-    roar_vio_printf(vio, "time: %u:%u\n", (int)((float)s->pos/(s->info.rate*s->info.channels)), plent->length);
+    roar_vio_printf(vio, "time: %u:%u\n", (int)((float)s->pos/(s->info.rate*s->info.channels)), (unsigned)plent->length);
     roar_vio_printf(vio, "audio: %i:%i:%i\n", s->info.rate, s->info.bits, s->info.channels);
    }
   } else {
