@@ -35,22 +35,22 @@ struct addqueue_para {
 
 static struct addqueue_para proto_simple_parse_addqueue(int client, char * txt);
 static int proto_simple_addqueue(int client, char * txt);
+static int proto_simple_serverinfo(int client);
+static int proto_simple_searchple(int client, char * args);
 
 // workarounds:
 #ifdef ROAR_TARGET_WIN32
 static int asprintf(char **strp, const char *fmt, ...) {
- static char buf[1024];
+ char buf[1024];
  int         ret;
  va_list     ap;
 
- // TODO: FIXME: this returns a static buffer, not a malloc()ed.
-
- *strp = buf;
-
  va_start(ap, fmt);
- ret = vsnprintf(buf, 1024, fmt, ap);
+ ret = vsnprintf(buf, sizeof(buf), fmt, ap);
  va_end(ap);
 
+ *strp = strdup(buf);
+
  return ret;
 }
 #endif
@@ -68,28 +68,66 @@ static void remove_spaces(char ** str) {
  *str = c;
 }
 
+static const char * __parse_string(char ** txt, const char * default_value) {
+ char quote = 0;
+ char * str;
+ char * ret;
+
+ remove_spaces(txt);
+
+ str = *txt;
+
+ if ( str[0] == '\'' || str[0] == '\"' ) {
+  quote = str[0];
+ } else {
+  if ( !strncasecmp(*txt, "default ", 8) ) {
+   *txt += 8;
+   return default_value;
+  } else if ( !strcasecmp(*txt, "default") ) {
+   *txt += 7;
+   return default_value;
+  }
+  roar_err_set(ROAR_ERROR_ILLSEQ);
+  return NULL;
+ }
+
+ str++;
+ ret = str;
+
+ while (*str && *str != quote) str++;
+
+ if ( *str == quote ) {
+  *str = 0;
+  *txt = str + 1;
+  return ret;
+ } else {
+  roar_err_set(ROAR_ERROR_ILLSEQ);
+  return NULL;
+ }
+}
+
 
 #define HT_BOOL          "TRUE|FALSE|TOGGLE"
-#define HT_PLI           "{\"Name\"|ID}"
-#define HT_PLI_ANY       "{\"Name\"|ID|ANY}"
+#define HT_PLI_RAW       "\"Name\"|ID"
+#define HT_PLI           "{" HT_PLI_RAW "}"
+#define HT_PLI_ANY_RAW   HT_PLI_RAW "|ANY"
+#define HT_PLI_ANY       "{" HT_PLI_ANY_RAW "}"
+#define HT_PLI_ANY_BUT   "{" HT_PLI_ANY_RAW " [BUT {QUEUES|HISTORIES}]...}"
 
-#ifdef HAVE_LIB_UUID
 #define HT_EI            "{long:0xLongID|0xLongID|short:0xShortID|uuid:UUID|pointer:name|num:Index|likeness:Likeness}"
-#else
-#define HT_EI            "{long:0xLongID|0xLongID|short:0xShortID|pointer:name|num:Index|likeness:Likeness}"
-#endif
 
 #define HT_IOI           "{STDIN|STDOUT|\"Filename\"}"
-#define HT_PLT           "{RPLD|PLS|M3U|VCLT|XSPF|PLAIN}"
+#define HT_PLT           "{RPLD|PLS|M3U|VCLT|XSPF|PLAIN|URAS}"
 #define HT_IMPORT_EXPORT "[" HT_PLI "] {TO|FROM} " HT_IOI " [AS " HT_PLT "]"
 #define HT_STORE_RESTORE "{NONE|CONFIG|ALL|QUEUE|PLAYLIST " HT_PLI_ANY "}"
 #define HT_PLE_BOPTS     HT_EI " [FROM " HT_PLI_ANY "]"
 #define HT_PLE_POS       " [AT POSITION]"
 #define HT_PLE_COPY_MOVE HT_PLE_BOPTS " TO " HT_PLI HT_PLE_POS
-#define HT_PLE_SEARCH_TERM "{\"search string\"|0xdiscid|uuid}"
-#define HT_PLE_SEARCH_F  "{ALL|TITLE|ARTIST|VERSION|FILENAME|DISCID|UUID|TAG:\"Tagname\"}"
-#define HT_PLE_SEARCH_OP "{AS|IN}"
-#define HT_PLE_SEARCH    HT_PLE_SEARCH_TERM " [" HT_PLE_SEARCH_OP " " HT_PLE_SEARCH_F "] [FROM " HT_PLI_ANY "]"
+#define HT_PLE_SEARCH_TERM "{\"search string\"|discid:0xdiscid|uuid:UUID|genre:genre|tracknum[ber]:num}"
+#define HT_PLE_SEARCH_F  "{ANY|ALBUM|TITLE|ARTIST|PERFORMER|VERSION|FILENAME|DISCID|UUID|GENRE|TRACKNUM[BER]|TAG:\"Tagname\"}"
+#define HT_PLE_SEARCH_OP "{{IS|AS}|IN|AT {BEGIN|END} OF}"
+#define HT_PLE_SEARCH_C  "[CASE[[ ]SENSITIVE]]"
+#define HT_PLE_SEARCH    HT_PLE_SEARCH_TERM " " HT_PLE_SEARCH_C " [NOT] " HT_PLE_SEARCH_OP " [NOT] " HT_PLE_SEARCH_F " [FROM " HT_PLI_ANY_BUT "]"
 #define HT_PTRI          "{CURRENT|DEFAULT|STARTUP|TEMP}"
 #define HT_POINTER_SET   HT_PTRI " [TO] " HT_PLE_BOPTS
 #define HT_POINTER_UNSET HT_PTRI
@@ -101,6 +139,7 @@ static char * proto_simple_help_text[] = {
  "Commands:\n",
  "  HELP\n",
  "  NOOP\n",
+ "  SERVERINFO\n",
  "  PLAY\n",
  "  STOP\n",
  "  NEXT\n",
@@ -140,6 +179,9 @@ static char * proto_simple_help_text[] = {
  "  SHOWQUEUE [" HT_PLI "]\n",
  "  ADDHISTORY [[TO] " HT_PLI "] [SIZE n]\n",
  "  DELHISTORY [" HT_PLI "]\n",
+ "  LISTCLIENTS\n",
+ "  IDENTIFY [WITH NAME name]\n", // we also support WITH {PID,NODENAME,HOSTID} but currently ignore those values as they are not stored.
+ "  SEARCHPLE " HT_PLE_SEARCH "\n",
  "Commands to fix:\n",
  "  SETPARENTLIST [OF " HT_PLI "] [TO] " HT_PLI "\n",
  "  SHOWPLAYING\n",
@@ -147,7 +189,6 @@ static char * proto_simple_help_text[] = {
  "  RESTORE " HT_STORE_RESTORE "\n",
  // TODO: To implement
  "Commands to implement:\n",
- "  SEARCHPLE " HT_PLE_SEARCH "\n",
  "  UPDATE " HT_IMPORT_EXPORT " USING " HT_PLE_SEARCH_F " [AND ...]" " [POLICY " HT_POLICYS " " HT_POLICY "]" "\n",
  "  AUTH [TO AUTHLEV {n|\"name\"}] USING authtype...\n",
  "Commands to remove (obsoleted):\n",
@@ -164,14 +205,14 @@ static int proto_simple_client_puts(int client, char * text) {
  return 0;
 }
 
-int proto_simple_client_handle(int id, struct roar_vio_calls * vio, struct roar_buffer ** obuffer, void ** userdata) {
+int proto_simple_client_handle(int id, struct roar_vio_calls * vio, struct roar_buffer ** obuffer, void ** userdata, const struct roar_keyval * para, ssize_t paralen, struct roar_dl_librarypara * pluginpara) {
  int status;
  ssize_t len;
  char buffer[1024];
  const char * statustext;
  const char * errortext = NULL;
 
- (void)obuffer, (void)userdata;
+ (void)obuffer, (void)userdata, (void)para, (void)paralen, (void)pluginpara;
 
  if ( vio == NULL )
   return -1;
@@ -190,7 +231,7 @@ int proto_simple_client_handle(int id, struct roar_vio_calls * vio, struct roar_
 
  buffer[len] = 0;
 
- if ( len > 0 ) {
+ if ( len > 0 && buffer[0] != '#' ) {
   status = proto_simple_parse(id, buffer, len);
  } else {
   status = PROTO_SIMPLE_STATUS_OK;
@@ -270,6 +311,10 @@ int proto_simple_parse(int client, char * line, size_t len) {
   }
 
   return PROTO_SIMPLE_STATUS_OK;
+ } else if ( !strcasecmp(cmd, "serverinfo") ) {
+  _CKPERM(ACCLEV_CONCTL);
+  _CKARGS(NULL_OK);
+  return proto_simple_serverinfo(client);
  } else if ( !strcasecmp(cmd, "quit") ) {
   _CKPERM(ACCLEV_CONCTL);
   _CKARGS(NULL_OK);
@@ -893,6 +938,11 @@ int proto_simple_parse(int client, char * line, size_t len) {
   _CKARGS(NULL_NOT_OK);
   return proto_simple_import_export(client, args, !strcasecmp(cmd, "export"));
 
+ } else if ( !strcasecmp(cmd, "searchple") ) {
+  _CKPERM(ACCLEV_USER);
+  _CKARGS(NULL_NOT_OK);
+  return proto_simple_searchple(client, args);
+
  } else if ( !strcasecmp(cmd, "store") ) {
   _CKPERM(ACCLEV_ALL);
   _CKARGS(NULL_OK);
@@ -922,6 +972,14 @@ int proto_simple_parse(int client, char * line, size_t len) {
   } else {
    return proto_simple_showpointer(client, args, 0);
   }
+ } else if ( !strcasecmp(cmd, "listclients") ) {
+  _CKPERM(ACCLEV_USER);
+  _CKARGS(NULL_OK);
+  return proto_simple_listclients(client);
+ } else if ( !strcasecmp(cmd, "identify") ) {
+  _CKPERM(ACCLEV_CONCTL);
+  _CKARGS(NULL_OK);
+  return proto_simple_identify(client, args);
  }
 
  // unknown command
@@ -930,21 +988,13 @@ int proto_simple_parse(int client, char * line, size_t len) {
 
 int proto_simple_print_plent(int client, const struct rpld_playlist_entry * plent) {
  char buffer[1024];
-#ifdef HAVE_LIB_UUID
  char uuid[37];
-#endif
 
-#ifdef HAVE_LIB_UUID
- uuid_unparse(plent->uuid, uuid);
-#endif
+ roar_uuid2str(uuid, plent->uuid, sizeof(uuid));
 
 //TYPE=LENGTH=ALBUM=TITLE=ARTIST=VERSION=FILE
- snprintf(buffer, 1023,
-#ifdef HAVE_LIB_UUID
-          "%s=%s=%s=%s=%s=%s=%s=%s=long:0x%.16LX/short:0x%.8X/uuid:%s=0x%.8x/%i=%s(0x%x)=%f\n",
-#else
-          "%s=%s=%s=%s=%s=%s=%s=%s=long:0x%.16LX/short:0x%.8X=0x%.8x/%i=%s(0x%x)=%f\n",
-#endif
+ snprintf(buffer, sizeof(buffer) - 1,
+          "%s=%s=%s=%s=%s=%s=%s=%s=long:0x%.16" __ll "X/short:0x%.8X/uuid:%s=0x%.8x/%i=%s(0x%x)=%f\n",
                         roar_codec2str(plent->codec),
                         rpld_ple_time_hr(plent),
                         plent->meta.album,
@@ -953,11 +1003,9 @@ int proto_simple_print_plent(int client, const struct rpld_playlist_entry * plen
                         plent->meta.performer,
                         plent->meta.version,
                         plent->io.filename,
-                        (long long unsigned int)plent->global_tracknum,
+                        (__longlong unsigned int)plent->global_tracknum,
                         plent->global_short_tracknum,
-#ifdef HAVE_LIB_UUID
                         uuid,
-#endif
                         plent->meta.discid,
                         plent->meta.tracknum,
                         roar_meta_strgenre(plent->meta.genre),
@@ -1039,7 +1087,7 @@ int proto_simple_print_playlist (int client, struct rpld_playlist * pl) {
  plq      = pl->queue;
 
  if ( histsize != -1 ) {
-  snprintf(options, sizeof(options), "history size: %zu", (size_t)histsize);
+  snprintf(options, sizeof(options), "history size: %" __ll "u", (__longlong unsigned int)histsize);
  } else if ( plq != NULL ) {
   snprintf(options, sizeof(options), "volume: %u/65535, role: %s, history: %i, backend: \"%s\", mixer: %i",
                                      (unsigned int)plq->volume_mono,
@@ -1064,7 +1112,7 @@ int proto_simple_parse_ple_bopts_noeval(int client, char * txt, int need_prev, i
  char * from;
  char * pli  = NULL;
  char   from_save = 0;
- long long unsigned int tmp;
+ __longlong unsigned int tmp;
 
  if ( txt == NULL || client == -1 )
   return -1;
@@ -1112,14 +1160,14 @@ int proto_simple_parse_ple_bopts_noeval(int client, char * txt, int need_prev, i
   if ( *ei == 'l' )
    ei += 5;
 
-  if ( sscanf(ei, "0x%16llX", &tmp) != 1 ) {
+  if ( sscanf(ei, "0x%16" __ll "X", &tmp) != 1 ) {
    pls->subject.long_tracknum = 0;
   } else {
    pls->subject.long_tracknum = tmp;
   }
 
   pls->type = RPLD_PL_ST_TRACKNUM_LONG;
-  ROAR_DBG("proto_simple_parse_ple_bopts(*): pls->subject.long_tracknum=0x%.16llX", pls->subject.long_tracknum);
+  ROAR_DBG("proto_simple_parse_ple_bopts(*): pls->subject.long_tracknum=0x%.16" __ll "X", (__longlong unsigned int)pls->subject.long_tracknum);
  } else if ( !strncmp(ei, "pointer:", 8) ) {
   pointer = rpld_pointer_get_by_name(ei+8, client);
 
@@ -1142,13 +1190,11 @@ int proto_simple_parse_ple_bopts_noeval(int client, char * txt, int need_prev, i
    rpld_plp_unref(pointer);
   }
 
-#ifdef HAVE_LIB_UUID
  } else if ( !strncmp(ei, "uuid:", 5) ) {
-  if ( uuid_parse(ei+5, pls->subject.uuid) != 0 )
-   uuid_clear(pls->subject.uuid);
+  if ( roar_str2uuid(pls->subject.uuid, ei+5) != 0 )
+   roar_uuid_clear(pls->subject.uuid);
 
   pls->type = RPLD_PL_ST_UUID;
-#endif
  } else if ( !strncmp(ei, "num:", 4) ) {
   pls->subject.num = (size_t)atoi(ei+4);
   pls->type = RPLD_PL_ST_NUM;
@@ -1388,7 +1434,7 @@ int proto_simple_showvolume (int client) {
  pli_t queue = client_get_queue(client);
  char buf[80];
 
- snprintf(buf, 80, "VOLUME %i/65535 %i%%\n", playback_get_volume_mu16(queue), playback_get_volume_mpc(queue));
+ snprintf(buf, sizeof(buf), "VOLUME %i/65535 %i%%\n", playback_get_volume_mu16(queue), playback_get_volume_mpc(queue));
 
  proto_simple_client_puts(client, buf);
 
@@ -1398,11 +1444,9 @@ int proto_simple_showvolume (int client) {
 #define _BS  80
 int proto_simple_showplaying (int client) {
  struct rpld_playlist_entry * plent;
- struct roar_stream         * s;
+ const struct roar_stream   * s;
  char buf[_BS];
-#ifdef HAVE_LIB_UUID
  char uuid[37];
-#endif
 
  if ( !playback_is_playing(client_get_queue(client)) ) {
   proto_simple_client_puts(client, "STATE STOPPED\n");
@@ -1421,18 +1465,16 @@ int proto_simple_showplaying (int client) {
   return PROTO_SIMPLE_STATUS_ERROR;
 
 /*
- *           "%s=%s=%s=%s=%s=%s=%s=%s=long:0x%.16LX/short:0x%.8X/uuid:%s=0x%.8x/%i=%s(0x%x)\n",
+ *           "%s=%s=%s=%s=%s=%s=%s=%s=long:0x%.16llX/short:0x%.8X/uuid:%s=0x%.8x/%i=%s(0x%x)\n",
  */
- snprintf(buf, _BS, "LONGGTN long:0x%.16LX\n", (long long unsigned int)plent->global_tracknum);
+ snprintf(buf, _BS, "LONGGTN long:0x%.16" __ll "X\n", (__longlong unsigned int)plent->global_tracknum);
  proto_simple_client_puts(client, buf);
  snprintf(buf, _BS, "SHORTGTN short:0x%.8X\n", plent->global_short_tracknum);
  proto_simple_client_puts(client, buf);
 
-#ifdef HAVE_LIB_UUID
- uuid_unparse(plent->uuid, uuid);
+ roar_uuid2str(uuid, plent->uuid, sizeof(uuid));
  snprintf(buf, _BS, "UUID uuid:%s\n", uuid);
  proto_simple_client_puts(client, buf);
-#endif
 
 // send meta data:
  if ( *(plent->meta.album) != 0 ) {
@@ -1538,15 +1580,9 @@ int proto_simple_setpointer(int client, char * args) {
 
   memset(&pls, 0, sizeof(pls));
 
-#ifdef HAVE_LIB_UUID
   pls.type = RPLD_PL_ST_UUID;
 
-  uuid_copy(pls.subject.uuid, ple->uuid);
-#else
-  pls.type = RPLD_PL_ST_TRACKNUM_LONG;
-
-  pls.subject.long_tracknum = ple->global_tracknum;
-#endif
+  memcpy(pls.subject.uuid, ple->uuid, sizeof(pls.subject.uuid));
  }
 
  plp = rpld_plp_init(NULL, &pls);
@@ -1570,9 +1606,7 @@ int proto_simple_showpointer(int client, char * name, int no_rename) {
  int pointer, pointer_client;
  struct rpld_playlist_pointer * plp;
  char * text = NULL;
-#ifdef HAVE_LIB_UUID
  char uuid[37];
-#endif
 
  if ( client == -1 || name == NULL || *name == 0 )
   return PROTO_SIMPLE_STATUS_ERROR;
@@ -1596,17 +1630,15 @@ int proto_simple_showpointer(int client, char * name, int no_rename) {
  } else {
   switch (plp->pls.type) {
    case RPLD_PL_ST_TRACKNUM_LONG:
-     asprintf(&text, "POINTER %s IS AT long:0x0x%.16LX\n", name, (long long unsigned int)plp->pls.subject.long_tracknum);
+     asprintf(&text, "POINTER %s IS AT long:0x0x%.16" __ll "X\n", name, (__longlong unsigned int)plp->pls.subject.long_tracknum);
     break;
    case RPLD_PL_ST_TRACKNUM_SHORT:
      asprintf(&text, "POINTER %s IS AT short:0x0x%.8X\n", name, plp->pls.subject.short_tracknum);
     break;
-#ifdef HAVE_LIB_UUID
    case RPLD_PL_ST_UUID:
-     uuid_unparse(plp->pls.subject.uuid, uuid);
+     roar_uuid2str(uuid, plp->pls.subject.uuid, sizeof(uuid));
      asprintf(&text, "POINTER %s IS AT uuid:%s\n", name, uuid);
     break;
-#endif
     case RPLD_PL_ST_RANDOM:
      asprintf(&text, "POINTER %s IS AT random:%i\n", name, plp->pls.subject.pl);
     break;
@@ -1668,59 +1700,6 @@ int proto_simple_showpointer_all(int client) {
  return PROTO_SIMPLE_STATUS_OK;
 }
 
-static char * __parse_string(char ** txt) {
- char quote = 0;
- char * str;
- char * ret;
-
- remove_spaces(txt);
-
- str = *txt;
-
- if ( str[0] == '\'' || str[0] == '\"' ) {
-  quote = str[0];
- } else {
-  roar_err_set(ROAR_ERROR_ILLSEQ);
-  return NULL;
- }
-
- str++;
- ret = str;
-
- while (*str && *str != quote) str++;
-
- if ( *str == quote ) {
-  *str = 0;
-  *txt = str + 1;
-  return ret;
- } else {
-  roar_err_set(ROAR_ERROR_ILLSEQ);
-  return NULL;
- }
-}
-
-static char * __parse_backend(char ** txt) {
- ROAR_DBG("__parse_backend(txt=%p{'%s'}) = ?", txt, *txt);
-
- if ( txt[0][0] == '\'' || txt[0][0] == '\"' )
-  return __parse_string(txt);
-
- ROAR_DBG("__parse_backend(txt=%p{'%s'}) = ?", txt, *txt);
-
- if ( !strncasecmp(*txt, "default ", 8) ) {
-  *txt += 8;
-  return "+default";
- }
- if ( !strcasecmp(*txt, "default") ) {
-  *txt += 7;
-  return "+default";
- }
-
- ROAR_DBG("__parse_backend(txt=%p{'%s'}) = NULL // error = ROAR_ERROR_ILLSEQ", txt, *txt);
- roar_err_set(ROAR_ERROR_ILLSEQ);
- return NULL;
-}
-
 static struct addqueue_para proto_simple_parse_addqueue(int client, char * txt) {
 //   ADDQUEUE [[TO] {"Name"|ID}] [WITH BACKEND backend] [MIXER n] WITH HISTORY {"Name"|ID} [WITH ROLE role]
  struct addqueue_para ret = {.queue = -1, .history = -1, .backend = "+default", .mixer = -1, .role = -1};
@@ -1762,7 +1741,7 @@ static struct addqueue_para proto_simple_parse_addqueue(int client, char * txt)
 
   if ( !strncasecmp(txt, "with backend ", 13) ) {
    txt += 13;
-   ret.backend = __parse_backend(&txt);
+   ret.backend = __parse_string(&txt, "+default");
    if ( ret.backend == NULL ) {
     ret.queue = -1;
     return ret;
@@ -1782,7 +1761,7 @@ static struct addqueue_para proto_simple_parse_addqueue(int client, char * txt)
    rpld_pl_unref(pl);
   } else if ( !strncasecmp(txt, "with role", 9) ) {
    txt += 9;
-   ret.role = roar_str2role(__parse_string(&txt));
+   ret.role = roar_str2role(__parse_string(&txt, "music"));
   } else {
    ret.history = ret.queue = -1;
    roar_err_set(ROAR_ERROR_ILLSEQ);
@@ -1850,4 +1829,298 @@ static int proto_simple_addqueue(int client, char * txt) {
  return PROTO_SIMPLE_STATUS_OK;
 }
 
+int proto_simple_listclients (int client) {
+ char buf[80];
+ int i;
+ int proto;
+ const char * name;
+
+ for (i = 0; i < RPLD_CLIENTS_MAX; i++) {
+  proto = client_get_proto(i);
+  if ( proto == RPLD_PROTO_UNUSED )
+   continue;
+
+  name = client_get_name(i);
+  if ( name == NULL )
+   name = "Nameless RPLD client";
+
+  snprintf(buf, sizeof(buf), "%3i: [protocol: \"%s\"] \"%s\"\n", i, client_proto2str(proto), name);
+  buf[sizeof(buf)-1] = 0;
+  proto_simple_client_puts(client, buf);
+ }
+
+ return PROTO_SIMPLE_STATUS_OK;
+}
+
+int proto_simple_identify(int client, char * txt) {
+ const char * name = NULL;
+ const char * nodename = NULL;
+ pid_t pid = 0;
+ long int hostid = 0;
+
+ remove_spaces(&txt);
+
+ while (txt != NULL && *txt) {
+  remove_spaces(&txt);
+
+  if ( !!strncasecmp(txt, "with ", 5) ) {
+   roar_err_set(ROAR_ERROR_ILLSEQ);
+   return PROTO_SIMPLE_STATUS_ERROR;
+  }
+
+  txt += 5;
+  remove_spaces(&txt);
+
+/*
+ X NAME
+ * PID
+ * NODENAME
+ * HOSTID
+ */
+
+  if ( !strncasecmp(txt, "name ", 5) ) {
+   txt += 5;
+   name = __parse_string(&txt, NULL);
+  } else if ( !strncasecmp(txt, "pid ", 4) ) {
+   txt += 4;
+   pid = strtol(txt, &txt, 0);
+  } else if ( !strncasecmp(txt, "nodename ", 9) ) {
+   txt += 9;
+   nodename = __parse_string(&txt, NULL);
+  } else if ( !strncasecmp(txt, "hostid ", 7) ) {
+   txt += 7;
+   hostid = strtol(txt, &txt, 0);
+  } else {
+   roar_err_set(ROAR_ERROR_BADRQC);
+   return PROTO_SIMPLE_STATUS_ERROR;
+  }
+ }
+
+ // avoid compiler warnings. Those will likely be used later.
+ _LIBROAR_IGNORE_RET(hostid);
+ _LIBROAR_IGNORE_RET(pid);
+ _LIBROAR_IGNORE_RET(nodename);
+
+ if ( name != NULL )
+  client_set_name(client, name);
+ return PROTO_SIMPLE_STATUS_OK;
+}
+
+static int proto_simple_serverinfo(int client) {
+ const struct roar_server_info * info = rpld_serverinfo_get();
+ char buf[1024];
+
+ proto_simple_client_puts(client, "X-APPLICATION \"" APPLICATION_NAME "\"\n");
+
+#define _display2(var,prefix) \
+ if ( info->var != NULL ) { \
+  snprintf(buf, sizeof(buf), prefix "\n", info->var); \
+  proto_simple_client_puts(client, buf); \
+ }
+#define _display(var,prefix) _display2(var,prefix " \"%s\"")
+ _display(version,     "VERSION")
+ _display(location,    "LOCATION")
+ _display(description, "DESCRIPTION")
+ _display(contact,     "CONTACT")
+ _display(serial,      "SERIAL")
+ _display(address,     "ADDRESS")
+ _display(uiurl,       "UIURL")
+ _display2(hostid,     "HOSTID %s")
+ _display(license,     "LICENSE")
+ _display(build,       "BUILD")
+ _display(un.sysname,  "SYSTEM NAME")
+ _display(un.release,  "SYSTEM RELEASE")
+ _display(un.nodename, "SYSTEM NODENAME")
+ _display(un.machine,  "SYSTEM MACHINE")
+#undef _display
+#undef _display2
+
+ return PROTO_SIMPLE_STATUS_OK;
+}
+
+static inline void __searchple_handle_playlist(int client, struct rpld_playlist * pl, const void * needle, const char * key, int options, int neg) {
+ const struct rpld_playlist_entry * plent = rpld_pl_get_first(pl);
+ int ret;
+
+ while (plent != NULL) {
+  ret = rpld_ple_filter(plent, needle, key, options);
+//  if ( (ret == 1 && !neg) || (ret == 0 && neg) )
+  if ( ret ^ neg )
+   proto_simple_print_plent(client, plent);
+  plent = plent->list.next;
+ }
+}
+
+static int proto_simple_searchple(int client, char * args) {
+ struct rpld_playlist       * pl = NULL;
+ const void * needle;
+ const char * key, * binkey = NULL;
+ int options = RPLD_PLE_FILTER_NORMKEY;
+ size_t i;
+ char * next;
+ int bneedle_si;
+ discid_t bneedle_discid;
+ roar_uuid_t bneedle_uuid;
+ int neg = 0;
+ int with_queues = 1, with_histories = 1;
+
+ remove_spaces(&args);
+ if ( *args == '"' || *args == '\'' ) {
+  needle = __parse_string(&args, NULL);
+ } else if ( !strncasecmp(args, "discid:", 7) ) {
+  args += 7;
+  binkey = "discid";
+  if ( sscanf(args, "0x%x", &bneedle_si) != 1 ) {
+   roar_err_set(ROAR_ERROR_ILLSEQ);
+   return PROTO_SIMPLE_STATUS_ERROR;
+  }
+  bneedle_discid = bneedle_si;
+  needle = &bneedle_discid;
+ } else if ( !strncasecmp(args, "uuid:", 5) ) {
+  args += 5;
+  binkey = "uuid";
+  if ( roar_str2uuid(bneedle_uuid, args) != 0 )
+   return PROTO_SIMPLE_STATUS_ERROR;
+  needle = &bneedle_uuid;
+ } else if ( !strncasecmp(args, "genre:", 6) ) {
+  args += 6;
+  binkey = "genre";
+  bneedle_si = roar_meta_intgenre(args);
+  needle = &bneedle_si;
+ } else if ( !strncasecmp(args, "tracknum:", 9) ) {
+  args += 9;
+  binkey = "tracknum";
+  bneedle_si = atoi(args);
+  needle = &bneedle_si;
+ } else if ( !strncasecmp(args, "tracknumber:", 12) ) {
+  args += 12;
+  binkey = "tracknumber";
+  bneedle_si = atoi(args);
+  needle = &bneedle_si;
+ } else {
+  roar_err_set(ROAR_ERROR_PROTO);
+  return PROTO_SIMPLE_STATUS_ERROR;
+ }
+
+ if ( binkey != NULL )
+  args = strstr(args, " ");
+
+ remove_spaces(&args);
+ if ( !strncasecmp(args, "case", 4) ) {
+  args += 4;
+  options |= RPLD_PLE_FILTER_CASE;
+  remove_spaces(&args);
+  if ( !strncasecmp(args, "sensitive", 9) ) {
+   args += 9;
+   remove_spaces(&args);
+  }
+ }
+
+ if ( !strncasecmp(args, "not ", 4) ) {
+  neg ^= 1;
+  args += 4;
+  remove_spaces(&args);
+ }
+
+ if ( !strncasecmp(args, "in", 2) ) {
+  args += 2;
+ } else if ( !strncasecmp(args, "as", 2) || !strncasecmp(args, "is", 2) ) {
+  args += 2;
+  options |= RPLD_PLE_FILTER_ANCHORB|RPLD_PLE_FILTER_ANCHORE;
+ } else if ( !strncasecmp(args, "at begin of", 11) ) {
+  args += 11;
+  options |= RPLD_PLE_FILTER_ANCHORB;
+ } else if ( !strncasecmp(args, "at end of", 9) ) {
+  args += 9;
+  options |= RPLD_PLE_FILTER_ANCHORE;
+ }
+ remove_spaces(&args);
+
+ if ( !strncasecmp(args, "not ", 4) ) {
+  neg ^= 1;
+  args += 4;
+  remove_spaces(&args);
+ }
+ remove_spaces(&args);
+
+ next = strstr(args, " ");
+ if ( next != NULL ) {
+  *next = 0;
+  next++;
+  remove_spaces(&next);
+ }
+
+ if ( !strcasecmp(args, "all") || !strcasecmp(args, "any") ) {
+  key = NULL;
+  options |= RPLD_PLE_FILTER_EXTKEY;
+ } else if ( !strncasecmp(args, "tag:", 4) ) {
+  args += 4;
+  key = __parse_string(&args, NULL);
+  options |= RPLD_PLE_FILTER_EXTKEY;
+ } else {
+  key = args;
+ }
+
+ if ( binkey != NULL ) {
+  if ( key == NULL ) {
+   key = binkey;
+  } if ( !!strcasecmp(key, binkey) ) {
+   roar_err_set(ROAR_ERROR_USER);
+   return PROTO_SIMPLE_STATUS_ERROR;
+  }
+  options |= RPLD_PLE_FILTER_BNEEDLE;
+ }
+
+ if ( next == NULL ) {
+  pl = rpld_pl_get_by_id(client_get_playlist(client));
+  if ( pl == NULL ) {
+   roar_err_set(ROAR_ERROR_NOENT);
+   return PROTO_SIMPLE_STATUS_OK;
+  }
+ } else {
+  if ( !strncasecmp(next, "from", 4) )
+   next += 4;
+   remove_spaces(&next);
+  if ( !strcasecmp(next, "any") || !strncasecmp(next, "any ", 4) ) {
+   pl = NULL;
+   next += 3;
+   remove_spaces(&next);
+   while ( !strncasecmp(next, "but ", 4) ) {
+    next += 4;
+    remove_spaces(&next);
+    if ( !strncasecmp(next, "queues", 6) ) {
+     with_queues = 0;
+     next += 6;
+    } else if ( !strncasecmp(next, "histories", 9) ) {
+     with_histories = 0;
+     next += 9;
+    }
+    remove_spaces(&next);
+   }
+  } else {
+   pl = proto_simple_parse_pli(next, NULL);
+   if ( pl == NULL ) {
+    roar_err_set(ROAR_ERROR_NOENT);
+    return PROTO_SIMPLE_STATUS_OK;
+   }
+  }
+ }
+
+ if ( pl != NULL ) {
+  __searchple_handle_playlist(client, pl, needle, key, options, neg);
+  rpld_pl_unref(pl);
+ } else {
+  for (i = 0; i < MAX_PLAYLISTS; i++) {
+   pl = rpld_pl_get_by_id(i);
+   if ( pl != NULL ) {
+    if ( (pl->queue == NULL || with_queues) && (pl->histsize == -1 || with_histories) )
+     __searchple_handle_playlist(client, pl, needle, key, options, neg);
+    rpld_pl_unref(pl);
+   }
+  }
+ }
+
+ return PROTO_SIMPLE_STATUS_OK;
+}
+
 //ll
