@@ -37,7 +37,7 @@
 #define PROTO_SIMPLE_STATUS_NOTPERM  7 /* not permitted */
 #define PROTO_SIMPLE_STATUS_NFPROERR 8 /* non fatal protocol error (syntax error?) */
 
-int proto_simple_client_handle(int id, struct roar_vio_calls * vio, struct roar_buffer ** obuffer, void ** userdata);
+int proto_simple_client_handle(int id, struct roar_vio_calls * vio, struct roar_buffer ** obuffer, void ** userdata, const struct roar_keyval * para, ssize_t paralen, struct roar_dl_librarypara * pluginpara);
 int proto_simple_parse(int client, char * line, size_t len);
 int proto_simple_print_plent(int client, const struct rpld_playlist_entry * plent);
 
@@ -63,6 +63,10 @@ int proto_simple_unsetpointer(int client, char * args);
 int proto_simple_showpointer(int client, char * name, int no_rename);
 int proto_simple_showpointer_all(int client);
 
+int proto_simple_listclients (int client);
+
+int proto_simple_identify(int client, char * txt);
+
 #endif
 
 //ll
