@@ -28,6 +28,8 @@
 int     rpld_proto_simple_vio_open    (struct roar_vio_calls * vio, struct roar_vio_calls * dst) {
  struct rpld_proto_simple_inst * self;
 
+ ROAR_DBG("rpld_proto_simple_vio_open(vio=%p, dst=%p) = ?", vio, dst);
+
  if ( vio == NULL || dst == NULL )
   return -1;
 
@@ -44,6 +46,8 @@ int     rpld_proto_simple_vio_open    (struct roar_vio_calls * vio, struct roar_
  self->output_dots = -1;
  self->eof         =  0;
 
+ vio->refc     = 1;
+ vio->flags    = ROAR_VIO_FLAGS_NONE;
  vio->inst     = self;
  vio->close    = rpld_proto_simple_vio_close;
  vio->read     = rpld_proto_simple_vio_read;
@@ -51,17 +55,21 @@ int     rpld_proto_simple_vio_open    (struct roar_vio_calls * vio, struct roar_
  vio->sync     = rpld_proto_simple_vio_sync;
 // vio->ctl      = rpld_proto_simple_vio_ctl;
 
+ ROAR_DBG("rpld_proto_simple_vio_open(vio=%p, dst=%p) = 0", vio, dst);
  return 0;
 }
 
 int     rpld_proto_simple_vio_close   (struct roar_vio_calls * vio) {
  struct rpld_proto_simple_inst * self = vio->inst;
 
+ ROAR_DBG("rpld_proto_simple_vio_close(vio=%p) = ?", vio);
+
  if ( self->iobuffer != NULL )
   roar_buffer_free(self->iobuffer);
 
  roar_mm_free(self);
 
+ ROAR_DBG("rpld_proto_simple_vio_close(vio=%p) = 0", vio);
  return 0;
 }
 
@@ -72,12 +80,12 @@ ssize_t rpld_proto_simple_vio_read    (struct roar_vio_calls * vio, void *buf, s
  ssize_t ret;
  void  * bufdata;
 
- ROAR_DBG("rpld_proto_simple_vio_read(vio=%p, buf=%p, count=%llu) = ?", vio, buf, (unsigned long long)count);
+ ROAR_DBG("rpld_proto_simple_vio_read(vio=%p, buf=%p, count=%lu) = ?", vio, buf, (unsigned long)count);
 
  if ( count == 0 )
   return 0;
 
- ROAR_DBG("rpld_proto_simple_vio_read(vio=%p, buf=%p, count=%llu) = ?", vio, buf, (unsigned long long)count);
+ ROAR_DBG("rpld_proto_simple_vio_read(vio=%p, buf=%p, count=%lu) = ?", vio, buf, (unsigned long)count);
 
  if ( self->iobuffer == NULL ) {
   if ( roar_buffer_new(&(self->iobuffer), RPLD_PROTO_SIMPLE_VIO_BUFSIZE) == -1 )
@@ -102,20 +110,20 @@ ssize_t rpld_proto_simple_vio_read    (struct roar_vio_calls * vio, void *buf, s
 
   _LIBROAR_IGNORE_RET(roar_buffer_set_offset(self->iobuffer, count));
 
- ROAR_DBG("rpld_proto_simple_vio_read(vio=%p, buf=%p, count=%llu) = %llu", vio, buf, (unsigned long long)count, (unsigned long long) count);
+ ROAR_DBG("rpld_proto_simple_vio_read(vio=%p, buf=%p, count=%lu) = %lu", vio, buf, (unsigned long)count, (unsigned long) count);
   return count;
  }
 
  ROAR_DBG("rpld_proto_simple_vio_read(*): entering main loop...");
 
  while ( done < count ) {
-  ROAR_DBG("rpld_proto_simple_vio_read(*): we still need %llu bytes", (unsigned long long) (count - done));
+  ROAR_DBG("rpld_proto_simple_vio_read(*): we still need %lu bytes", (unsigned long) (count - done));
 
   if ( roar_buffer_get_len(self->iobuffer, &buflen) == -1 )
    return -1;
 
   if ( buflen ) {
-   ROAR_DBG("rpld_proto_simple_vio_read(*): we got %llu bytes from buffer", (unsigned long long) buflen);
+   ROAR_DBG("rpld_proto_simple_vio_read(*): we got %lu bytes from buffer", (unsigned long) buflen);
 
    if ( roar_buffer_get_data(self->iobuffer, &bufdata) == -1 )
     break;
@@ -147,9 +155,9 @@ ssize_t rpld_proto_simple_vio_read    (struct roar_vio_calls * vio, void *buf, s
 
   ret = roar_vio_read(self->vio, bufdata, RPLD_PROTO_SIMPLE_VIO_BUFSIZE);
 
-  ROAR_DBG("rpld_proto_simple_vio_read(*): got %lli bytes from network", (long long) ret);
+  ROAR_DBG("rpld_proto_simple_vio_read(*): got %li bytes from network", (long) ret);
 
-  if ( ret == -1 )
+  if ( ret == -1 || ret == 0 )
    break;
 
   if ( self->input_dots == 0 ) {
@@ -182,7 +190,7 @@ ssize_t rpld_proto_simple_vio_read    (struct roar_vio_calls * vio, void *buf, s
    return -1;
  }
 
- ROAR_DBG("rpld_proto_simple_vio_read(vio=%p, buf=%p, count=%llu) = %llu", vio, buf, (unsigned long long)count, (unsigned long long) done);
+ ROAR_DBG("rpld_proto_simple_vio_read(vio=%p, buf=%p, count=%lu) = %lu", vio, buf, (unsigned long)count, (unsigned long) done);
 
  return done;
 }
