@@ -25,6 +25,20 @@
 
 #include "rpld.h"
 
+#define DEFAULT_ROLE ROAR_ROLE_MUSIC
+
+enum iodir {
+ IODIR_IMPORT,
+ IODIR_EXPORT
+};
+
+struct import_playlist {
+ const char * playlist;
+ const char * type;
+ const char * filename;
+ enum iodir   iodir;
+};
+
 #if defined(ROAR_HAVE_IO_POSIX) && defined(ROAR_HAVE_FS_POSIX)
 #define SUPPORT_PIDFILE
 static const char * pidfile = NULL;
@@ -46,11 +60,22 @@ void usage (const char * progname) {
  printf(
         " -h --help               - Show this help\n"
         "    --daemon             - Daemonize\n"
-        "    --server SERVER      - Connect to roard at SERVER\n"
+        "    --watchdog           - Enable watchdog\n"
+        "    --server SERVER      - Connect to the RoarAudio server at SERVER\n"
+        "    --server-queue SERV  - The server to use for the Main Queue.\n"
+        "                           Normally --server should be used.\n"
+        "    --server-info  SERV  - Copy server infos from this server.\n"
 #ifdef SUPPORT_PIDFILE
         "    --pidfile PIDFILE    - Write a pidfile at PIDFILE\n"
 #endif
         "    --histsize NUM       - Set history size to NUM\n"
+        "    --location  LOC      - Set lion readable location of server\n"
+        "    --description  DESC  - Set lion readable description of server\n"
+        "    --contact CONTACT    - Set contact for this server\n"
+        "    --serial SERIAL      - Set serial for this device or server\n"
+        "                           (for embedded devices only)\n"
+        "    --uiurl UIURL        - Set URL for userinterface of this device or server\n"
+        "                           (for embedded devices only)\n"
 #ifdef SUPPORT_PLUGIN
         "    --plugin-load PLUGIN - Load plugin PLUGIN\n"
         "    --plugin-args ARGS   - Arguments for the plugin\n"
@@ -81,6 +106,7 @@ void usage (const char * progname) {
  printf(
         "    --play               - Start in playling state\n"
         "    --stopped            - Start in stopped state (default)\n"
+        "    --no-default-queue   - Do not create a default queue\n"
         "    --role ROLE          - Set role for streams (default: music)\n"
         "    --codec-helper-rate RATE\n"
         "    --codec-helper-bits BITS\n"
@@ -91,8 +117,10 @@ void usage (const char * progname) {
 
  printf("\nPlayList Options:\n\n");
  printf(
-        "    --load-playlist PLAYLIST TYPE FILE\n"
+        "    --playlist-import PLAYLIST TYPE FILE\n"
         "                         - Load playlist PLAYLIST of type TYPE from file FILE\n"
+        "    --playlist-export PLAYLIST TYPE FILE\n"
+        "                         - export playlist PLAYLIST to file FILE in format TYPE\n"
        );
 
  printf("\nNetwork Options:\n\n");
@@ -209,7 +237,7 @@ static void addstartup(void) {
  }
 }
 
-static int loadplaylist(const char * playlist, const char * type, const char * file) {
+static int imexportplaylist(const struct import_playlist * playlist) {
  struct roar_vio_defaults def;
  struct fformat_handle  * handle;
  struct roar_vio_calls    vio;
@@ -217,51 +245,57 @@ static int loadplaylist(const char * playlist, const char * type, const char * f
  int format;
  int ret;
 
- if ( playlist == NULL || *playlist == 0 || type == NULL || *type == 0 || file == NULL || *file == 0 ) {
-  ROAR_ERR("loadplaylist(playlist='%s', type='%s', file='%s'): Invalid argument", playlist, type, file);
+ if ( playlist->playlist == NULL || playlist->playlist[0] == 0 ||
+      playlist->type     == NULL || playlist->type[0]     == 0 ||
+      playlist->filename == NULL || playlist->filename[0] == 0 ) {
+  ROAR_ERR("imexportplaylist(playlist='%s', type='%s', file='%s'): Invalid argument", playlist->playlist, playlist->type, playlist->filename);
   return -1;
  }
 
- if ( roar_vio_dstr_init_defaults(&def, ROAR_VIO_DEF_TYPE_NONE, O_RDONLY, 0644) == -1 ) {
-  ROAR_ERR("loadplaylist(playlist='%s', type='%s', file='%s'): Can not create DSTR defaults", playlist, type, file);
+ if ( roar_vio_dstr_init_defaults(&def, ROAR_VIO_DEF_TYPE_NONE, playlist->iodir == IODIR_IMPORT ? O_RDONLY : O_WRONLY|O_CREAT|O_TRUNC, 0644) == -1 ) {
+  ROAR_ERR("imexportplaylist(playlist='%s', type='%s', file='%s'): Can not create DSTR defaults", playlist->playlist, playlist->type, playlist->filename);
   return -1;
  }
 
- format = fformat_str2plf((char*)type);
+ format = fformat_str2plf(playlist->type);
  if ( format == -1 ) {
-  ROAR_ERR("loadplaylist(playlist='%s', type='%s', file='%s'): Unknown playlist type", playlist, type, file);
+  ROAR_ERR("imexportplaylist(playlist='%s', type='%s', file='%s'): Unknown playlist type", playlist->playlist, playlist->type, playlist->filename);
   return -1;
  }
 
  handle = fformat_handle_new(format);
  if ( handle == NULL ) {
-  ROAR_ERR("loadplaylist(playlist='%s', type='%s', file='%s'): Can not open file format handle for playlist type", playlist, type, file);
+  ROAR_ERR("imexportplaylist(playlist='%s', type='%s', file='%s'): Can not open file format handle for playlist type", playlist->playlist, playlist->type, playlist->filename);
   return -1;
  }
 
- if ( roar_vio_open_dstr(&vio, (char*)file, &def, 1) == -1 ) {
-  ROAR_ERR("loadplaylist(playlist='%s', type='%s', file='%s'): Can not open file: %s", playlist, type, file, roar_error2str(roar_error));
+ if ( roar_vio_open_dstr(&vio, playlist->filename, &def, 1) == -1 ) {
+  ROAR_ERR("imexportplaylist(playlist='%s', type='%s', file='%s'): Can not open file: %s", playlist->playlist, playlist->type, playlist->filename, roar_error2str(roar_error));
   fformat_handle_close(handle);
   return -1;
  }
 
- pl = rpld_pl_get_by_name(playlist);
+ pl = rpld_pl_get_by_name(playlist->playlist);
  if ( pl == NULL ) {
   pl = rpld_pl_new();
   if ( pl != NULL ) {
-   rpld_pl_set_name(pl, playlist);
+   rpld_pl_set_name(pl, playlist->playlist);
    rpld_pl_register(pl);
   } 
  }
 
  if ( pl == NULL ) {
-  ROAR_ERR("loadplaylist(playlist='%s', type='%s', file='%s'): Playlist not found and playlist creation failed", playlist, type, file);
+  ROAR_ERR("imexportplaylist(playlist='%s', type='%s', file='%s'): Playlist not found and playlist creation failed", playlist->playlist, playlist->type, playlist->filename);
   roar_vio_close(&vio);
   fformat_handle_close(handle);
   return -1;
  }
 
- ret = fformat_playlist_import(handle, &vio, pl);
+ if ( playlist->iodir == IODIR_IMPORT ) {
+  ret = fformat_playlist_import(handle, &vio, pl);
+ } else {
+  ret = fformat_playlist_export(handle, &vio, pl);
+ }
 
  roar_vio_close(&vio);
 
@@ -270,12 +304,14 @@ static int loadplaylist(const char * playlist, const char * type, const char * f
  rpld_pl_unref(pl);
 
  if ( ret == -1 ) {
-  ROAR_ERR("loadplaylist(playlist='%s', type='%s', file='%s'): Can not import playlist", playlist, type, file);
+  ROAR_ERR("imexportplaylist(playlist='%s', type='%s', file='%s'): Can not import playlist", playlist->playlist, playlist->type, playlist->filename);
  }
  return ret;
 }
 
 int main (int argc, char * argv[]) {
+ struct roar_server_info serverinfo;
+ const char * info_server = NULL;
  struct roar_audio_info * codechelper_info = playback_get_codechelper_info();
  struct roar_vio_calls * clientvio;
  struct rpld_playlist * queue, * history;
@@ -283,26 +319,43 @@ int main (int argc, char * argv[]) {
  int do_restore  = 0;
  int do_store    = 0;
  int i;
- char * k;
- char * n_server = NULL;
+ const char * k;
+ const char * n_server = NULL;
  int    n_listen =  1;
  int    n_port   = -1;
  int    n_type   = ROAR_SOCKET_TYPE_UNKNOWN;
- int    n_proto  = RPLD_PROTO_SIMPLE;
+ int    n_proto  = ROAR_PROTO_RPLD;
  enum rpld_client_acclev n_acclev = ACCLEV_ALL;
  int    playing  = 0;
 #ifdef HAVE_LIB_SLP
  int    slp      = 0;
 #endif
  int    daemon   = 0;
+ int    watchdog = 0;
  int is_playing  = 1;
- int role;
+ const char * queue_server = NULL;
+ int role = DEFAULT_ROLE;
+ int create_default_queue = 1;
 #ifdef SUPPORT_PLUGIN
  const char * plugin_args = NULL;
 #endif
+ struct import_playlist import_playlist[8];
+ int    import_num = 0;
 
  ROAR_DBG("main(*) = ?");
 
+ memset(&serverinfo, 0, sizeof(serverinfo));
+
+ if ( roar_notify_core_new_global(-1) == -1 ) {
+  ROAR_ERR("main(*): Can not create new global notify bus: %s", roar_error2str(roar_error));
+  return 1;
+ }
+
+ if ( roar_notify_core_register_proxy(NULL, roar_notify_proxy_std, NULL) == -1 ) {
+  ROAR_ERR("main(*): Can not register global notify proxy: %s", roar_error2str(roar_error));
+  return 1;
+ }
+
  if ( backend_init() == -1 ) {
   ROAR_ERR("main(*): Can not start backend module: %s", roar_error2str(roar_error));
   return 1;
@@ -330,7 +383,6 @@ int main (int argc, char * argv[]) {
  rpld_pl_set_name(queue,   "Main Queue");
  rpld_pl_set_name(history, "Main History");
  rpld_pl_set_histsize(history, 42);
- playback_add_queue(queue, NULL /* FIXME */, -1 /* FIXME */, history);
 
  ROAR_DBG("main(*) = ?");
 
@@ -354,8 +406,14 @@ int main (int argc, char * argv[]) {
    return 0;
   } else if ( !strcmp(k, "--daemon") ) {
    daemon = 1;
+  } else if ( !strcmp(k, "--watchdog") ) {
+   watchdog = 1;
   } else if ( !strcmp(k, "--server") ) {
    roar_libroar_set_server(argv[++i]);
+  } else if ( !strcmp(k, "--server-queue") ) {
+   queue_server = argv[++i];
+  } else if ( !strcmp(k, "--server-info") ) {
+   info_server = argv[++i];
   } else if ( strcmp(k, "--pidfile") == 0 ) {
 #ifdef SUPPORT_PIDFILE
    pidfile = argv[++i];
@@ -364,6 +422,17 @@ int main (int argc, char * argv[]) {
    i++;
 #endif
 
+  } else if ( strcmp(k, "--location") == 0 ) {
+   serverinfo.location = argv[++i];
+  } else if ( strcmp(k, "--description") == 0 ) {
+   serverinfo.description = argv[++i];
+  } else if ( strcmp(k, "--contact") == 0 ) {
+   serverinfo.contact = argv[++i];
+  } else if ( strcmp(k, "--serial") == 0 ) {
+   serverinfo.serial = argv[++i];
+  } else if ( strcmp(k, "--uiurl") == 0 ) {
+   serverinfo.uiurl = argv[++i];
+
   } else if ( strcmp(k, "--plugin-args") == 0 ) {
 #ifdef SUPPORT_PLUGIN
    plugin_args = argv[++i];
@@ -400,7 +469,12 @@ int main (int argc, char * argv[]) {
 #endif
 
   } else if ( !strcmp(k, "--histsize") ) {
-   rpld_pl_set_histsize(history, atol(argv[++i]));
+   i++;
+   if ( !strcmp(argv[i], "default") ) {
+    // noop. Used by rpld-storemgr to simplify the argument generator.
+   } else {
+    rpld_pl_set_histsize(history, atol(argv[i]));
+   }
 
   } else if ( !strcmp(k, "--user") ) {
 #if defined(ROAR_HAVE_SETUID) && defined(ROAR_HAVE_IO_POSIX)
@@ -453,12 +527,18 @@ int main (int argc, char * argv[]) {
    playing = 1;
   } else if ( !strcmp(k, "--stopped") ) {
    playing = 0;
+  } else if ( !strcmp(k, "--no-default-queue") ) {
+   create_default_queue = 0;
   } else if ( !strcmp(k, "--role") ) {
-   if ( (role = roar_str2role(argv[++i])) == -1 ) {
-    fprintf(stderr, "Error: can not set role (unknown role name?) to: %s\n", argv[i]);
-    return 1;
+   i++;
+   if ( !strcmp(argv[i], "default") ) {
+    role = DEFAULT_ROLE;
+   } else {
+    if ( (role = roar_str2role(argv[i])) == -1 ) {
+     fprintf(stderr, "Error: can not set role (unknown role name?) to: %s\n", argv[i]);
+     return 1;
+    }
    }
-   playback_set_role(rpld_pl_get_id(queue), role);
 
   } else if ( !strcmp(k, "--codec-helper-rate") ) {
    codechelper_info->rate = roar_str2rate(argv[++i]);
@@ -469,10 +549,19 @@ int main (int argc, char * argv[]) {
   } else if ( !strcmp(k, "--codec-helper-codec") ) {
    codechelper_info->codec = roar_str2codec(argv[++i]);
 
-  } else if ( !strcmp(k, "--load-playlist") ) {
-   if ( loadplaylist(argv[i+1], argv[i+2], argv[i+3]) == -1 ) {
-    return 1;
+  // --load-playlist is old parameter name and should no longer be used.
+  } else if ( !strcmp(k, "--load-playlist") || !strcmp(k, "--playlist-import") || !strcmp(k, "--playlist-export") ) {
+   if ( import_num == (sizeof(import_playlist)/sizeof(*import_playlist)) ) {
+    ROAR_WARN("Can not import more playlists: maximum number of playlists to import reached.");
+    i += 3;
+    continue;
    }
+
+   import_playlist[import_num].playlist = argv[i+1];
+   import_playlist[import_num].type     = argv[i+2];
+   import_playlist[import_num].filename = argv[i+3];
+   import_playlist[import_num].iodir    = !strcmp(k, "--playlist-export") ? IODIR_EXPORT : IODIR_IMPORT;
+   import_num++;
    i += 3;
 
   } else if ( !strcmp(k, "--tcp") ) {
@@ -499,10 +588,8 @@ int main (int argc, char * argv[]) {
    fprintf(stderr, "Warning: No OpenSLP support compiled in.\n");
 #endif
 
-/* TODO: FIXME
   } else if ( !strcmp(k, "--bl-codec") ) {
-   playback_blacklist_codec(roar_str2codec(argv[++i]));
-*/
+   backend_blacklist_codec_global(roar_str2codec(argv[++i]));
   } else {
    fprintf(stderr, "Error: Invalid option: %s\n", k);
    usage(argv[0]);
@@ -524,6 +611,20 @@ int main (int argc, char * argv[]) {
  }
 #endif
 
+ // set up the main queue.
+ if ( create_default_queue ) {
+  playback_add_queue(queue, queue_server, -1 /* FIXME */, history);
+  playback_set_role(rpld_pl_get_id(queue), role);
+ }
+
+ rpld_pl_unref(queue);
+ rpld_pl_unref(history);
+
+ // setting up server info:
+ if ( info_server != NULL )
+  rpld_serverinfo_update(RPLD_SERVERINFO_SOURCE_SERVER, info_server);
+ rpld_serverinfo_update(RPLD_SERVERINFO_SOURCE_OBJECT, &serverinfo);
+
  if ( playback_init() == -1 )
   return 1;
 
@@ -539,6 +640,12 @@ int main (int argc, char * argv[]) {
   }
  }
 
+ for (i = 0; i < import_num; i++) {
+  if ( imexportplaylist(&(import_playlist[i])) == -1 ) {
+   ROAR_WARN("Failed to import/export playlist: %s: %s", import_playlist[i].filename, roar_error2str(roar_error));
+  }
+ }
+
 // We need to fork before the init the playback backend
 // so the PID does not change after the init.
 // We will close the STD* handles later.
@@ -559,6 +666,9 @@ int main (int argc, char * argv[]) {
 #endif
  }
 
+ yield_watchdog_active(watchdog);
+ yield_watchdog_start();
+
  ROAR_DBG("main(*) = ?");
 
  if ( n_listen )
@@ -621,9 +731,6 @@ int main (int argc, char * argv[]) {
 
  addstartup();
 
- if ( playing )
-  playback_play_all();
-
 // Now we should have everything up and running, we close the STD* handles.
  if ( daemon ) {
   roar_debug_set_stderr_mode(ROAR_DEBUG_MODE_SYSLOG);
@@ -638,8 +745,12 @@ int main (int argc, char * argv[]) {
   rpld_plugin_appsched(ROAR_DL_APPSCHED_INIT);
 #endif
 
+ if ( playing )
+  playback_play_all();
+
  while (g_alive) {
   // main loop, check clients, play files
+  yield_watchdog_trigger();
   network_check(is_playing ? RPLD_YIELD_BLOCK : RPLD_YIELD_LONGBLOCK);
   playback_check(RPLD_YIELD_BLOCK);
   is_playing = playback_is_playing_any();
@@ -673,9 +784,6 @@ int main (int argc, char * argv[]) {
 
  playback_uninit();
 
- rpld_pl_unref(queue);
- rpld_pl_unref(history);
-
  backend_free();
 
  if ( pidfile != NULL )
