@@ -34,10 +34,8 @@
 #include <roaraudio.h>
 #include <time.h>
 
-#ifdef HAVE_LIB_UUID
-#include <uuid/uuid.h>
-#endif
-
+#include "config.h"
+#include "rpldnotify.h"
 #include "playlist.h"
 #include "fformat.h"
 #include "backend.h"
@@ -53,8 +51,9 @@
 #include "rpldslp.h"
 #include "pointer.h"
 #include "plugin.h"
+#include "serverinfo.h"
 
-#define APPLICATION_NAME "RoarAudio PlayList daemon"
+#define APPLICATION_NAME "RoarAudio PlayList Daemon"
 
 #if defined(ROAR_HAVE_SETGID) && defined(ROAR_HAVE_IO_POSIX)
 extern struct group   * g_grp;
@@ -63,6 +62,14 @@ extern struct group   * g_grp;
 extern struct passwd  * g_pwd;
 #endif
 
+#ifdef ROAR_TARGET_WIN32
+#define __longlong long
+#define __ll       "l"
+#else
+#define __longlong long long
+#define __ll       "ll"
+#endif
+
 void rpld_exit(void);
 
 #endif
