@@ -29,7 +29,7 @@
 #include <roaraudio.h>
 
 #define RPLDPH_APPNAME    "rpld " ROAR_VSTR_ROARAUDIO
-#define RPLDPH_ABIVERSION "0.1rc7"
+#define RPLDPH_ABIVERSION "0.1.5"
 
 #define RPLDPH_CHECK_VERSIONS() ROAR_DL_PLUGIN_CHECK_VERSIONS(RPLDPH_APPNAME, RPLDPH_ABIVERSION)
 
@@ -43,6 +43,7 @@ typedef unsigned int rpld_pli_t;
 #define RPLD_PLI_INVALID ((rpld_pli_t)-1)
 struct rpld_fformat_handle;
 typedef struct rpld_fformat_handle * rpld_fformat_t;
+typedef int64_t rpld_maxint_t;
 
 typedef enum {
  RPLDPH_CONFIG_KEY_MAX_CLIENTS = 16,
@@ -90,6 +91,7 @@ struct __rpldph_functions_error {
 
 struct __rpldph_functions_control {
  void (*_exit)(void);
+ const struct roar_server_info * (*_serverinfo)(void);
 };
 
 struct __rpldph_functions_ple {
@@ -99,6 +101,8 @@ struct __rpldph_functions_ple {
  int                          (*_set_res)     (struct rpld_playlist_entry * plent, const char * file, int codec, time_t length);
  int                          (*_set_meta)    (struct rpld_playlist_entry * plent, struct roar_keyval * kv, ssize_t len);
  const char *                 (*_get_meta)    (struct rpld_playlist_entry * plent, const char * key);
+ int                          (*_set_meta_int)(struct rpld_playlist_entry * plent, const char * key, rpld_maxint_t val);
+ rpld_maxint_t                (*_get_meta_int)(struct rpld_playlist_entry * plent, const char * key);
  int                          (*_get_codec)   (struct rpld_playlist_entry * plent);
  time_t                       (*_get_length)  (struct rpld_playlist_entry * plent);
  int                          (*_set_uuid)    (struct rpld_playlist_entry * plent, const char * uuid);
@@ -107,6 +111,8 @@ struct __rpldph_functions_ple {
  int                          (*_inc_likeness)(struct rpld_playlist_entry * plent, float likeness);
  float                        (*_get_likeness)(struct rpld_playlist_entry * plent);
  struct rpld_playlist_entry * (*_get_next)    (struct rpld_playlist_entry * plent);
+ int                          (*_set_offset)  (struct rpld_playlist_entry * plent, int_least32_t   start, int_least32_t   end);
+ int                          (*_get_offset)  (struct rpld_playlist_entry * plent, int_least32_t * start, int_least32_t * end);
 };
 
 struct __rpldph_functions_playlist {
@@ -136,7 +142,7 @@ struct __rpldph_functions_history {
 };
 
 struct __rpldph_functions_plp {
- struct rpld_playlist_pointer * (*_new)     (rpld_pli_t playlist, const char * type, void * argp);
+ struct rpld_playlist_pointer * (*_new)     (rpld_pli_t playlist, const char * type, const void * argp);
  int                            (*_ref)     (struct rpld_playlist_pointer * plp);
  int                            (*_unref)   (struct rpld_playlist_pointer * plp);
  int                            (*_copy)    (struct rpld_playlist_pointer ** dst,
@@ -162,6 +168,7 @@ struct __rpldph_functions_playback {
 
  struct rpld_playlist_entry * (*_get_ple)(rpld_pli_t queue);
  size_t (*_get_mduc)(rpld_pli_t queue);
+ const struct roar_stream * (*_get_stream)(rpld_pli_t queue);
 };
 
 struct __rpldph_functions_store {
@@ -176,8 +183,10 @@ struct __rpldph_functions_backend {
  int (*_unref)(struct rpld_backend * backend);
 
  const char * (*_get_name)(struct rpld_backend * backend);
+ struct roar_connection * (*_get_con)(struct rpld_backend * backend); // need to roar_connectionunref() after use.
 
  int (*_blacklist_codec)(struct rpld_backend * backend, int codec);
+ int (*_blacklist_codec_global)(int codec);
  int (*_blacklist_check_codec)(struct rpld_backend * backend, int codec);
 };
 
@@ -186,12 +195,16 @@ struct __rpldph_functions_client {
 // int  client_set_acclev(int id, enum rpld_client_acclev acclev, int setter);
 // int  client_inc_acclev(int id, int inc, int setter);
 
+ int  (*_set_name)(int id, const char * name);
+ const char * (*_get_name)(int id);
+
  int  (*_set_playlist)(int id, rpld_pli_t pl);
  rpld_pli_t (*_get_playlist)(int id);
  int  (*_set_queue)(int id, rpld_pli_t pl);
  rpld_pli_t (*_get_queue)(int id);
 
  int  (*_str2proto)(const char * str);
+ const char * (*_proto2str)(const int proto);
 // enum rpld_client_acclev (*_str2acclev)(const char * str);
 };
 
@@ -226,6 +239,7 @@ struct __rpldph_functions_yield {
  void (*_ay_trigger)(void);
  void (*_ay_pause)(void);
  void (*_ay_unpause)(void);
+ void (*_watchdog_trigger)(void);
 };
 
 struct __rpldph_functions {
@@ -291,6 +305,7 @@ struct __rpldph_binargv {
 #define rpldph_error_error2str(arg) __rpldph_export1(error,_error2str,arg)
 
 #define rpldph_control_exit() __rpldph_export0(control,_exit)
+#define rpldph_control_serverinfo() __rpldph_export0(control,_serverinfo)
 
 #define rpldph_ple_new()     __rpldph_export0(ple,_new)
 #define rpldph_ple_free(arg) __rpldph_export1(ple,_free,arg)
@@ -298,6 +313,8 @@ struct __rpldph_binargv {
 #define rpldph_ple_set_res(arg,...) __rpldph_exportn(ple,_set_res,arg,__VA_ARGS__)
 #define rpldph_ple_set_meta(arg,...) __rpldph_exportn(ple,_set_meta,arg,__VA_ARGS__)
 #define rpldph_ple_get_meta(arg,...) __rpldph_exportn(ple,_get_meta,arg,__VA_ARGS__)
+#define rpldph_ple_set_meta_int(arg,...) __rpldph_exportn(ple,_set_meta_int,arg,__VA_ARGS__)
+#define rpldph_ple_get_meta_int(arg,...) __rpldph_exportn(ple,_get_meta_int,arg,__VA_ARGS__)
 #define rpldph_ple_get_codec(arg) __rpldph_export1(ple,_get_codec,arg)
 #define rpldph_ple_get_time(arg) __rpldph_export1(ple,_get_time,arg)
 #define rpldph_ple_set_uuid(arg,...) __rpldph_exportn(ple,_set_uuid,arg,__VA_ARGS__)
@@ -306,6 +323,8 @@ struct __rpldph_binargv {
 #define rpldph_ple_inc_likeness(arg,...) __rpldph_exportn(ple,_inc_likeness,arg,__VA_ARGS__)
 #define rpldph_ple_get_likeness(arg) __rpldph_export1(ple,_get_likeness,arg)
 #define rpldph_ple_get_next(arg) __rpldph_export1(ple,_get_next,arg)
+#define rpldph_ple_set_offset(arg,...) __rpldph_exportn(ple,_set_offset,arg,__VA_ARGS__)
+#define rpldph_ple_get_offset(arg,...) __rpldph_exportn(ple,_get_offset,arg,__VA_ARGS__)
 
 #define rpldph_playlist_new(arg,...) __rpldph_exportn(playlist,_new,arg,__VA_ARGS__)
 #define rpldph_playlist_free(arg) __rpldph_export1(playlist,_free,arg)
@@ -322,7 +341,7 @@ struct __rpldph_binargv {
 #define rpldph_playlist_pop(arg,...) __rpldph_exportn(playlist,_pop,arg,__VA_ARGS__)
 #define rpldph_playlist_shift(arg,...) __rpldph_exportn(playlist,_shift,arg,__VA_ARGS__)
 #define rpldph_playlist_add(arg,...) __rpldph_exportn(playlist,_add,arg,__VA_ARGS__)
-#define rpldph_playlist_get_first(arg,...) __rpldph_exportn(playlist,_get_first,arg,__VA_ARGS__)
+#define rpldph_playlist_get_first(arg) __rpldph_export1(playlist,_get_first,arg)
 
 #define rpldph_history_set_size(arg,...) __rpldph_exportn(history,_set_size,arg,__VA_ARGS__)
 #define rpldph_history_get_size(arg) __rpldph_export1(history,_get_size,arg)
@@ -348,6 +367,7 @@ struct __rpldph_binargv {
 
 #define rpldph_playback_get_ple(arg) __rpldph_export1(playback,_get_ple,arg)
 #define rpldph_playback_get_mduc(arg) __rpldph_export1(playback,_get_mduc,arg)
+#define rpldph_playback_get_stream(arg) __rpldph_export1(playback,_get_stream,arg)
 
 #define rpldph_store_save() __rpldph_export0(store,_save)
 #define rpldph_store_load() __rpldph_export0(store,_load)
@@ -356,9 +376,19 @@ struct __rpldph_binargv {
 #define rpldph_backend_ref(arg) __rpldph_export1(backend,_ref,arg)
 #define rpldph_backend_unref(arg) __rpldph_export1(backend,_unref,arg)
 #define rpldph_backend_get_name(arg) __rpldph_export1(backend,_get_name,arg)
+#define rpldph_backend_get_con(arg) __rpldph_export1(backend,_get_con,arg)
 #define rpldph_backend_blacklist_codec(arg,...) __rpldph_exportn(backend,_blacklist_codec,arg,__VA_ARGS__)
 #define rpldph_backend_blacklist_check_codec(arg,...) __rpldph_exportn(backend,_blacklist_codec,arg,__VA_ARGS__)
 
+#define rpldph_client_set_name(arg,...) __rpldph_exportn(client,_set_name,arg,__VA_ARGS__)
+#define rpldph_client_get_name(arg) __rpldph_export1(client,_get_name,arg)
+#define rpldph_client_set_playlist(arg,...) __rpldph_exportn(client,_set_playlist,arg,__VA_ARGS__)
+#define rpldph_client_get_playlist(arg) __rpldph_export1(client,_get_playlist,arg)
+#define rpldph_client_set_queue(arg,...) __rpldph_exportn(client,_set_queue,arg,__VA_ARGS__)
+#define rpldph_client_get_queue(arg) __rpldph_export1(client,_get_queue,arg)
+#define rpldph_client_str2proto(arg) __rpldph_export1(client,_str2proto,arg)
+#define rpldph_client_proto2str(arg) __rpldph_export1(client,_proto2str,arg)
+
 #define rpldph_fformat_str2plf(arg) __rpldph_export1(fformat,_str2plf,arg)
 #define rpldph_fformat_plf2str(arg) __rpldph_export1(fformat,_plf2str,arg)
 #define rpldph_fformat_handle_new(arg) __rpldph_export1(fformat,_handle_new,arg)
@@ -378,6 +408,7 @@ struct __rpldph_binargv {
 #define rpldph_yield_ay_trigger() __rpldph_export0(yield,_ay_trigger)
 #define rpldph_yield_ay_pause() __rpldph_export0(yield,_ay_pause)
 #define rpldph_yield_ay_unpause() __rpldph_export0(yield,_ay_unpause)
+#define rpldph_yield_watchdog_trigger() __rpldph_export0(yield,_watchdog_trigger)
 
 #endif
 
