@@ -31,7 +31,7 @@
 
 #ifdef ROAR_HAVE_LIBSLP
 void register_slp_callback(SLPHandle hslp, SLPError errcode, void * cookie);
-int register_slp (int unreg, char * sockname);
+int register_slp (int unreg, const char * sockname);
 #endif
 
 #endif
