@@ -71,9 +71,7 @@ int store (void) {
  size_t i;
  char * buf;
  size_t len;
-#ifdef HAVE_LIB_UUID
  char uuid[37];
-#endif
 
  // TODO: free format...
 
@@ -124,19 +122,17 @@ int store (void) {
 
    switch (plp->pls.type) {
     case RPLD_PL_ST_TRACKNUM_LONG:
-      roar_vio_printf(&file, "POINTER: \"%s\" -> long:0x0x%.16LX\n", pointer_name,
-                                plp->pls.subject.long_tracknum);
+      roar_vio_printf(&file, "POINTER: \"%s\" -> long:0x0x%.16" __ll "X\n", pointer_name,
+                                (__longlong unsigned int)plp->pls.subject.long_tracknum);
      break;
     case RPLD_PL_ST_TRACKNUM_SHORT:
-      roar_vio_printf(&file, "POINTER: \"%s\" -> short:0x0x%.8X\n", pointer_name,
-                                plp->pls.subject.short_tracknum);
+      roar_vio_printf(&file, "POINTER: \"%s\" -> short:0x0x%.8lX\n", pointer_name,
+                                (long unsigned int)plp->pls.subject.short_tracknum);
      break;
-#ifdef HAVE_LIB_UUID
     case RPLD_PL_ST_UUID:
-      uuid_unparse(plp->pls.subject.uuid, uuid);
+      roar_uuid2str(uuid, plp->pls.subject.uuid, sizeof(uuid));
       roar_vio_printf(&file, "POINTER: \"%s\" -> uuid:%s\n", pointer_name, uuid);
      break;
-#endif
     case RPLD_PL_ST_RANDOM:
       roar_vio_printf(&file, "POINTER: \"%s\" -> random:%i\n", pointer_name,
                                 plp->pls.subject.pl);
@@ -349,17 +345,12 @@ int restore_pointers (const char * filename, struct roar_vio_defaults * def) {
    }
 
    if ( !strcmp(div[1], "uuid") ) {
-#ifdef HAVE_LIB_UUID
     ROAR_DBG("restore(): uuid='%s'", div[2]);
     pls.type = RPLD_PL_ST_UUID;
-    if ( uuid_parse(div[2], pls.subject.uuid) != 0 )
+    if ( roar_str2uuid(pls.subject.uuid, div[2]) != 0 )
      continue;
 
     ROAR_DBG("restore() = ?");
-#else
-    ROAR_WARN("restore(): Unsupported search schema: %s", div[1]);
-    continue;
-#endif
    } else if ( !strcmp(div[1], "random") ) {
     pls.type = RPLD_PL_ST_RANDOM;
     pls.subject.pl = atoi(div[2]);
@@ -387,6 +378,7 @@ int restore_pointers (const char * filename, struct roar_vio_defaults * def) {
 int restore (void) {
  struct fformat_handle * format;
  struct rpld_playlist * pl, * history;
+ struct rpld_backend * backend;
  struct roar_vio_calls file;
  struct roar_vio_calls index, import;
  struct roar_vio_defaults def;
@@ -497,7 +489,21 @@ int restore (void) {
      history = rpld_pl_new();
      rpld_pl_register_id(history, index_lion.history);
     }
+
+    // first pre-connect to the backend server.
+    // this allows us to track connection errors a bit more easy.
+    backend = backend_get_connection2(index_lion.backend_name, 1);
+    if ( backend == NULL ) {
+     ROAR_WARN("restore(void): Can not create backend object for server. BAD.: %s: %s", index_lion.backend_name, roar_errorstring);
+    } else {
+     if ( !(backend->flags & BACKEND_FLAG_CONNECTED) ) {
+      ROAR_WARN("restore(void): Unabled to connect to server: %s: %s", index_lion.backend_name, roar_errorstring);
+     }
+     backend_unref(backend);
+    }
+
     playback_add_queue(pl, index_lion.backend_name, index_lion.mixer_id, history);
+
     playback_set_volume_mu16(index_lion.id, index_lion.volume_mono);
     playback_set_role(index_lion.id, index_lion.role);
     rpld_pl_unref(history);
