@@ -32,6 +32,7 @@
 #define RPLD_YIELD_TASK_PLAYBACK      0x0002
 #define RPLD_YIELD_TASK_PLUGIN_UPDATE 0x0004
 #define RPLD_YIELD_TASK_PLUGIN_TICK   0x0008
+#define RPLD_YIELD_TASK_WATCHDOG      0x0010
 
 #define RPLD_YIELD_NONBLOCK          0
 #define RPLD_YIELD_BLOCK             1
@@ -54,6 +55,11 @@ void yield(int ptasks, int ntasks, int block);
 
 void yield_auto(enum yield_auto cmd, int ptasks, int ntasks, int block);
 
+void yield_watchdog_active(int active);
+void yield_watchdog_start(void);
+void yield_watchdog_stop(void);
+void yield_watchdog_trigger(void);
+
 #endif
 
 //ll
